/**********************************************************************
*
*                            XgData_Edit.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*    Edit data set display parameters, or delete data set
*
***********************************************************************
*/
#ifdef sgi
#define USE_DIRENT
#endif
#ifdef __hpux
#define USE_DIRENT
#endif

#include "stdio.h"
#include "string.h"

/* either use the next line and then skip 2, or under strict system V,
   skip the next line and use the following 2 */
#ifndef USE_DIRENT
#include "sys/dir.h"
#else
#include "sys/types.h"
#include "dirent.h"
#endif

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "Xm/Form.h"
#include "Xm/Frame.h"
#include "Xm/SelectioB.h"
#include "Xm/SeparatoG.h"
#include "Xm/PushB.h"
#include "Xm/RowColumn.h"
#include "Xm/ToggleB.h"
#include "Xm/ToggleBG.h"
#include "Xm/PanedW.h"
#include "Xm/Text.h"
#include "Xm/List.h"
#include "Xm/FileSB.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

/*******************************************************************/
void Xg_select_data_set(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i,
    nargs;
  char
    buf[80];
  Widget
    subwidget,
    data_select;
  Arg
    args[10];
  XmString
    title,
    datasets_label,
    DataList[XgNSETS];

  XgGraphInfo
    *graph;
  XgWidStruct
    *wid_addr;

wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;

if ((graph->num_sets < 2) && ((int) wid_addr->client == XgDSelDelete))
  {
    Xg_error_sys(XgERR_NO_DELETE_LAST, graph);
    return;
  }
if (graph->cursor_status == XgCursOn)  /* turn off cursor */
  Xg_delcurs(graph);

for (nargs=0; nargs<graph->num_sets; ++nargs)
  {
    sprintf(buf, "%d %s", nargs, graph->dataset[nargs].filename);
    DataList[nargs] = XmStringCreateLtoR(buf, XgCharset);
  }
if (graph->num_sets > 5)  /** set number of files to show w/out scrollbars **/
  i = 5;
else
  i = graph->num_sets;
datasets_label = XmStringCreateLtoR("Available Data Files:", XgCharset);
title = XmStringCreateLtoR("Xg: Active Data Sets", XgCharset);

nargs = 0;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNlistItems, DataList); ++nargs;
XtSetArg(args[nargs], XmNlistItemCount, graph->num_sets); ++nargs;
XtSetArg(args[nargs], XmNlistLabelString, datasets_label); ++nargs;
XtSetArg(args[nargs], XmNlistVisibleItemCount, i); ++nargs;
XtSetArg(args[nargs], XmNtextString, DataList[0]); ++nargs;
XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
data_select = XmCreateSelectionDialog(graph->app_shell,
              "Xg: Active Data Sets", args, nargs);
subwidget = XmSelectionBoxGetChild(data_select, XmDIALOG_TEXT);
XtUnmanageChild(subwidget);
subwidget = XmSelectionBoxGetChild(data_select, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(subwidget);
subwidget = XmSelectionBoxGetChild(data_select, XmDIALOG_APPLY_BUTTON);
XtUnmanageChild(subwidget);

XtManageChild(data_select);
XtAddCallback(data_select, XmNhelpCallback, Xg_help_sys,
	      (caddr_t) XgHELP_DATA_SELECT);
if ((int) wid_addr->client == XgDSelDelete)
  {
    XtAddCallback(data_select, XmNokCallback, Xg_delete_dataset, graph);
  }
else if ((int) wid_addr->client == XgDSelSave)
  {
    XtAddCallback(data_select, XmNokCallback, Xg_save_dataset, graph);
  }
else
  XtAddCallback(data_select, XmNokCallback, Xg_edit_dataset, graph);
XtAddCallback(data_select, XmNcancelCallback, Xg_cancel_CB, NULL);
XtFree(datasets_label);
XtFree(title);

for (nargs=0; nargs<graph->num_sets; ++nargs)
  XtFree(DataList[nargs]);

}

/***************************/
void Xg_delete_dataset(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i,
    j,
    dataset;
  char
    buf[80],
    *filename;
  XmSelectionBoxCallbackStruct
    *select_rslt;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;

select_rslt = (XmSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(select_rslt->value, XgCharset, &filename);
sscanf(filename, "%d", &dataset);
XtFree(select_rslt);
free(filename);
XtDestroyWidget(w);

j = 0;
for (i=0; i<graph->num_sets; ++i)
  if (graph->dataset[i].left_right == XgLeft)
    ++j;
if ((j <= 1) && (graph->dataset[dataset].left_right == XgLeft))
  {
    Xg_error_sys(XgERR_LAST_Y_LEFT, graph);
    return;
  }

--graph->num_sets;
strcpy(graph->dataset[dataset].filename,
       graph->dataset[graph->num_sets].filename);
graph->dataset[dataset].num_points
  = graph->dataset[graph->num_sets].num_points;

free(graph->dataset[dataset].darray);
if (graph->dataset[dataset].err_array != NULL)
  free(graph->dataset[dataset].err_array);
graph->dataset[dataset].darray
  = graph->dataset[graph->num_sets].darray;
graph->dataset[dataset].err_array
  = graph->dataset[graph->num_sets].err_array;

graph->dataset[dataset].point_type
  = graph->dataset[graph->num_sets].point_type;
graph->dataset[dataset].point_size
  = graph->dataset[graph->num_sets].point_size;
graph->dataset[dataset].filled_open
  = graph->dataset[graph->num_sets].filled_open;
graph->dataset[dataset].line_status
  = graph->dataset[graph->num_sets].line_status;
graph->dataset[dataset].error_pts
  = graph->dataset[graph->num_sets].error_pts;
graph->dataset[dataset].error_bars
  = graph->dataset[graph->num_sets].error_bars;
graph->dataset[dataset].left_right
  = graph->dataset[graph->num_sets].left_right;

graph->dataset[dataset].in_legend
  = graph->dataset[graph->num_sets].in_legend;
strcpy(graph->dataset[dataset].legend_text,
       graph->dataset[graph->num_sets].legend_text);

strcpy(graph->dataset[dataset].description.text,
       graph->dataset[graph->num_sets].description.text);
graph->dataset[dataset].description.coord.x
  = graph->dataset[graph->num_sets].description.coord.x;
graph->dataset[dataset].description.coord.y
  = graph->dataset[graph->num_sets].description.coord.y;
graph->dataset[dataset].description.dimen.x
  = graph->dataset[graph->num_sets].description.dimen.x;
graph->dataset[dataset].description.dimen.y
  = graph->dataset[graph->num_sets].description.dimen.y;
graph->dataset[dataset].description.color
  = graph->dataset[graph->num_sets].description.color;
/*if (dataset != graph->num_sets)
  XtFree(graph->dataset[dataset].description.font_info.font_struct);*/
graph->dataset[dataset].description.font_info.font_struct
  = graph->dataset[graph->num_sets].description.font_info.font_struct;
strcpy(graph->dataset[dataset].description.font_info.name,
       graph->dataset[graph->num_sets].description.font_info.name);
}

/***************************/
void Xg_save_dataset(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i,
    j,
    nargs,
    dataset;
  char
    buf[80],
    *filename;
  XmSelectionBoxCallbackStruct
    *select_rslt;
  Arg
    args[10];
  XmString
    title;
  Widget
    file_selector;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;

select_rslt = (XmSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(select_rslt->value, XgCharset, &filename);
sscanf(filename, "%d", &dataset);
XtFree(select_rslt);
free(filename);
XtDestroyWidget(w);
graph->ftype = dataset;  /** save number of dataset to be saved **/

nargs = 0;
title = XmStringCreateLtoR("Xg: Save Data File", XgCharset);
/*XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;*/
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNdefaultPosition, False); ++nargs;
XtSetArg(args[nargs], XmNx, 400); ++nargs;
XtSetArg(args[nargs], XmNy, 100); ++nargs;
file_selector = XmCreateFileSelectionDialog(graph->app_shell,
					   "Xg: Save Data File", args, nargs);
XtManageChild(file_selector);
XtAddCallback(file_selector, XmNhelpCallback, Xg_help_sys,
	      (caddr_t) XgHELP_SAVE_FILE);
XtAddCallback(file_selector, XmNokCallback, Xg_save_file, graph);
XtAddCallback(file_selector, XmNcancelCallback, Xg_cancel_CB, NULL);
XtFree(title);
}

/***************************/
void Xg_save_file(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i;
  char
    *buf;
  FILE
    *fp;
  XmFileSelectionBoxCallbackStruct
    *dialog_rslt;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;

dialog_rslt = (XmFileSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(dialog_rslt->value, XgCharset, &buf);
strcpy(graph->dataset[graph->num_sets].filename, buf);
XtFree(dialog_rslt);
free(buf);

fp = fopen(graph->dataset[graph->num_sets].filename, "w");
if (fp == NULL)
  {
    Xg_error_sys(XgERR_NO_FILE, graph);
    return;
  }
XtDestroyWidget(w);
fprintf(fp, "%s\n", graph->dataset[graph->ftype].filename);
fprintf(fp, "%s\n", graph->dataset[graph->ftype].description.text);
fprintf(fp, "%d T\n", graph->dataset[graph->ftype].num_points);
for (i=0; i<graph->dataset[graph->ftype].num_points; ++i)
  {
    fprintf(fp, "%lg %lg\n", *(graph->dataset[graph->ftype].darray + 2*i),
	    *(graph->dataset[graph->ftype].darray + 2*i + 1));
  }
fclose(fp);
}

/***************************/
void Xg_edit_dataset(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i,
    nargs,
    dataset;
  char
    buf[80],
    *filename;
  Arg
    args[10];
  Widget
    parent,
    sb_form,
    label_frame,
    desc_frame,
    legend_frame,
    main_pts_frame,
    form,
    text_frame,
    font_frame,
    x_frame,
    y_frame,
    label,
    separ,
    text_wid,
    radio,
    point_type_frame,
    line_frame,
    point_size_frame,
    choices_frame,
    button;
  XmString
    frame_label,
    title,
    defaults;
  XmSelectionBoxCallbackStruct
    *select_rslt;

  XgGraphInfo
    *graph;

/*********** get parameters **************/
graph = (XgGraphInfo *) client_data;

select_rslt = (XmSelectionBoxCallbackStruct *) call_data;
XmStringGetLtoR(select_rslt->value, XgCharset, &filename);
sscanf(filename, "%d", &dataset);
XtFree(select_rslt);
graph->temp_data.num_points = dataset;

/********** load temp parameters **************/
strcpy(graph->temp_text, graph->dataset[dataset].description.text);
strcpy(graph->temp_font,
       graph->dataset[dataset].description.font_info.name);
graph->temp_coord.x = graph->dataset[dataset].description.coord.x;
graph->temp_coord.y = graph->dataset[dataset].description.coord.y;

strcpy(graph->temp_data.legend_text, graph->dataset[dataset].legend_text);
graph->temp_data.in_legend = graph->dataset[dataset].in_legend;

graph->temp_data.point_type = graph->dataset[dataset].point_type;
graph->temp_data.point_size = graph->dataset[dataset].point_size;
graph->temp_data.filled_open = graph->dataset[dataset].filled_open;
graph->temp_data.line_status = graph->dataset[dataset].line_status;
graph->temp_data.left_right = graph->dataset[dataset].left_right;
graph->temp_data.error_bars = graph->dataset[dataset].error_bars;
graph->temp_data.error_pts = graph->dataset[dataset].error_pts;

/********** main popup dialog shell ************/
nargs = 0;
defaults = XmStringCreateLtoR("Defaults", XgCharset);
title = XmStringCreateLtoR("Xg: Data Edit", XgCharset);
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNapplyLabelString, defaults); ++nargs;
XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
/*XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;*/
parent = XmCreatePromptDialog(graph->app_shell, "titles", args, nargs);
button = XmSelectionBoxGetChild(parent, XmDIALOG_TEXT);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(parent, XmDIALOG_APPLY_BUTTON);
XtManageChild(button);

nargs = 0;
sb_form = XmCreateForm(parent, "sb form", args, nargs);
XtManageChild(sb_form);

/********** label frame ****************/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
label_frame = XmCreateFrame(sb_form, "main label frame", args, nargs);
XtManageChild(label_frame);

  nargs = 0;
  sprintf(buf, "Dataset: %s\n", filename);
  frame_label = XmStringCreateLtoR(buf, XgCharset);
  XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
  label = XmCreateLabel(label_frame, "Title Label", args, nargs);
  XtManageChild(label);
  XtFree(frame_label);
  free(filename);

/********** description frame **********/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, label_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
desc_frame = XmCreateFrame(sb_form, "desc frame", args, nargs);
XtManageChild(desc_frame);

nargs = 0;
form = XmCreateForm(desc_frame, "desc form", args, nargs);
XtManageChild(form);

  /** Text Title Box **/
  nargs = 0;
  XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  text_frame = XmCreateFrame(form, "text frame", args, nargs);
  XtManageChild(text_frame);

  nargs = 0;
  button = XmCreateForm(text_frame, "desc text form", args, nargs);
  XtManageChild(button);

  nargs = 0;
  frame_label = XmStringCreateLtoR("Description", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
  XtSetArg(args[nargs], XmNalignment, XmALIGNMENT_BEGINNING); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  label = XmCreateLabel(button, "text Label", args, nargs);
  XtManageChild(label);
  XtFree(frame_label);

  nargs = 0;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, label); ++nargs;
  text_wid = XmCreateText(button, "text box", args, nargs);
  XmTextSetString(text_wid, graph->temp_text);
  XtManageChild(text_wid);
  XtAddCallback(text_wid, XmNvalueChangedCallback, Xg_get_temp_text, graph);

  /** Font Name Box **/
  nargs = 0;
  XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, text_frame); ++nargs;
  XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
  font_frame = XmCreateFrame(form, "text frame", args, nargs);
  XtManageChild(font_frame);

  nargs = 0;
  x_frame = XmCreateForm(font_frame, "font form", args, nargs);
  XtManageChild(x_frame);

  nargs = 0;
  sprintf(buf, "Font: %s", graph->temp_font);
  frame_label = XmStringCreateLtoR(buf, XgCharset);
  XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNbottomAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  graph->temp_widget = XmCreateLabel(x_frame, "font Label", args, nargs);
  XtManageChild(graph->temp_widget);
  XtFree(frame_label);

  nargs = 0;
  /*XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
    XtSetArg(args[nargs], XmNleftWidget, graph->temp_widget); ++nargs;
    XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;*/
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNbottomAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  button = XmCreatePushButton(x_frame, "Change", args, nargs);
  XtManageChild(button);
  XtAddCallback(button, XmNactivateCallback, Xg_choose_new_font, graph);

  /** X Coord Text Box **/
  nargs = 0;
  XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, font_frame); ++nargs;
  XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
  x_frame = XmCreateFrame(form, "x frame", args, nargs);
  XtManageChild(x_frame);

  nargs = 0;
  XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
  XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
  button = XmCreatePanedWindow(x_frame, "x window", args, nargs);
  XtManageChild(button);
  nargs = 0;
  frame_label = XmStringCreateLtoR("X Coordinate", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
  label = XmCreateLabel(button, "x Label", args, nargs);
  XtManageChild(label);
  XtFree(frame_label);

  nargs = 0;
  text_wid = XmCreateText(button, "x text", args, nargs);
  sprintf(buf, "%lg", graph->temp_coord.x);
  XmTextSetString(text_wid, buf);
  XtManageChild(text_wid);
  graph->wid_array[0].client = (caddr_t) XgX_pos;
  XtAddCallback(text_wid, XmNvalueChangedCallback, Xg_get_temp_coord,
		&graph->wid_array[0]);

  /** Y Coord Text Box **/
  nargs = 0;
  XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNleftWidget, x_frame); ++nargs;
  XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, font_frame); ++nargs;
  XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
  y_frame = XmCreateFrame(form, "y frame", args, nargs);
  XtManageChild(y_frame);

  nargs = 0;
  XtSetArg(args[nargs], XmNsashShadowThickness, 0); ++nargs;
  XtSetArg(args[nargs], XmNsashWidth, 1); ++nargs;
  button = XmCreatePanedWindow(y_frame, "y window", args, nargs);
  XtManageChild(button);
  nargs = 0;
  frame_label = XmStringCreateLtoR("Y Coordinate", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
  label = XmCreateLabel(button, "y Label", args, nargs);
  XtManageChild(label);
  XtFree(frame_label);

  nargs = 0;
  text_wid = XmCreateText(button, "y text", args, nargs);
  sprintf(buf, "%lg", graph->temp_coord.y);
  XmTextSetString(text_wid, buf);
  XtManageChild(text_wid);
  graph->wid_array[1].client = (caddr_t) XgY_pos;
  XtAddCallback(text_wid, XmNvalueChangedCallback, Xg_get_temp_coord,
		&graph->wid_array[1]);

/*********** Legend Frame ***********/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, desc_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
legend_frame = XmCreateFrame(sb_form, "legend frame", args, nargs);
XtManageChild(legend_frame);

nargs = 0;
form = XmCreateForm(legend_frame, "legend form", args, nargs);
XtManageChild(form);

  /** Text Title Box **/
  nargs = 0;
  XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  text_frame = XmCreateFrame(form, "legend text frame", args, nargs);
  XtManageChild(text_frame);

  nargs = 0;
  button = XmCreateForm(text_frame, "legend text form", args, nargs);
  XtManageChild(button);

  nargs = 0;
  frame_label = XmStringCreateLtoR("Legend description", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  label = XmCreateLabel(button, "text Label", args, nargs);
  XtManageChild(label);
  XtFree(frame_label);

  nargs = 0;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, label); ++nargs;
  text_wid = XmCreateText(button, "text box", args, nargs);
  XmTextSetString(text_wid, graph->temp_data.legend_text);
  XtManageChild(text_wid);
  XtAddCallback(text_wid, XmNvalueChangedCallback, Xg_get_temp_legend, graph);

  /**** in/out of legend radio box ****/
  nargs = 0;
  if (graph->temp_data.in_legend == XgInLegend)
    {
      XtSetArg(args[nargs], XmNset, True); ++nargs;
      frame_label = XmStringCreateLtoR("IN/out of Legend", XgCharset);
    }
  else
    {
      XtSetArg(args[nargs], XmNset, False); ++nargs;
      frame_label = XmStringCreateLtoR("in/OUT of Legend", XgCharset);
    }
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, text_frame); ++nargs;
  XtSetArg(args[nargs], XmNalignment, XmALIGNMENT_BEGINNING); ++nargs;
  XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
  XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
  button = XmCreateToggleButtonGadget(form, "In/Out of Legend", args, nargs);
  graph->wid_array[2].client = (caddr_t) XgInLegend;
  XtAddCallback(button, XmNvalueChangedCallback, Xg_toggle_data,
		&graph->wid_array[2]);
  XtManageChild(button);
  XtFree(frame_label);

/***** Data Points, Lines, Size, Filled/Open, Left/Right, Error Bars *****/
nargs = 0;
XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
XtSetArg(args[nargs], XmNtopWidget, legend_frame); ++nargs;
XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
main_pts_frame = XmCreateFrame(sb_form, "points/lines frame", args, nargs);
XtManageChild(main_pts_frame);

nargs = 0;
form = XmCreateForm(main_pts_frame, "data toggles form", args, nargs);
XtManageChild(form);

  /** Point Style Selector Radio Box **/
  nargs = 0;
  XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  point_type_frame = XmCreateFrame(form, "point type frame", args, nargs);
  XtManageChild(point_type_frame);

  nargs = 0;
  button = XmCreateForm(point_type_frame, "point type form", args, nargs);
  XtManageChild(button);

  nargs = 0;
  frame_label = XmStringCreateLtoR("Point Type", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  label = XmCreateLabel(button, "point label", args, nargs);
  XtManageChild(label);
  XtFree(frame_label);

  nargs = 0;
  XtSetArg(args[nargs], XmNseparatorType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, label); ++nargs;
/*  XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
  XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
  separ = XmCreateSeparatorGadget(button, "Point Type Sep", args, nargs);
  XtManageChild(separ);

  nargs = 0;
  XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, separ); ++nargs;
  XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
  radio = XmCreateRadioBox(button, "Point Type", args, nargs);
  XtManageChild(radio);
/*  XmAddTabGroup(radio);*/

      nargs = 0;
      if (graph->temp_data.point_type == XgPoint)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Point", args, nargs);
      graph->wid_array[4].client = (caddr_t) XgPoint;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[4]);
      XtManageChild(button);
      nargs = 0;
      if (graph->temp_data.point_type == XgCircle)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Circle", args, nargs);
      graph->wid_array[5].client = (caddr_t) XgCircle;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[5]);
      XtManageChild(button);
      nargs = 0;
      if (graph->temp_data.point_type == XgSquare)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Square    ", args, nargs);
      graph->wid_array[6].client = (caddr_t) XgSquare;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[6]);
      XtManageChild(button);
      nargs = 0;
      if (graph->temp_data.point_type == XgTriangle)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Triangle", args, nargs);
      graph->wid_array[7].client = (caddr_t) XgTriangle;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[7]);
      XtManageChild(button);
      nargs = 0;
      if (graph->temp_data.point_type == XgPlus)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Plus +", args, nargs);
      graph->wid_array[8].client = (caddr_t) XgPlus;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[8]);
      XtManageChild(button);
      nargs = 0;
      if (graph->temp_data.point_type == XgEx)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "X", args, nargs);
      graph->wid_array[9].client = (caddr_t) XgEx;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[9]);
      XtManageChild(button);
      nargs = 0;
      if (graph->temp_data.point_type == XgBar)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Bar", args, nargs);
      graph->wid_array[10].client = (caddr_t) XgBar;
      XtAddCallback(button,XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[10]);
      XtManageChild(button);

  /** Line Type Selector Radio Box **/
  nargs = 0;
  XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNleftWidget, point_type_frame); ++nargs;
  XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;
/*  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;*/
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  line_frame = XmCreateFrame(form, "line frame", args, nargs);
  XtManageChild(line_frame);

  nargs = 0;
  button = XmCreateForm(line_frame, "line form", args, nargs);
  XtManageChild(button);

  nargs = 0;
  frame_label = XmStringCreateLtoR("Line Type", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  label = XmCreateLabel(button, "line label", args, nargs);
  XtManageChild(label);
  XtFree(frame_label);

  nargs = 0;
  XtSetArg(args[nargs], XmNseparatorType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, label); ++nargs;
/*  XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
  XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
  separ = XmCreateSeparatorGadget(button, "Line Type Sep", args, nargs);
  XtManageChild(separ);

  nargs = 0;
  XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, separ); ++nargs;
  XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
  radio = XmCreateRadioBox(button, "line radio", args, nargs);
  XtManageChild(radio);
/*  XmAddTabGroup(radio);*/

      nargs = 0;
      if (graph->temp_data.line_status == XgNoLine)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "No Line", args, nargs);
      graph->wid_array[11].client = (caddr_t) XgNoLine;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[11]);
      XtManageChild(button);
      nargs = 0;
      if (graph->temp_data.line_status == XgSolidLine)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Solid Line", args, nargs);
      graph->wid_array[12].client = (caddr_t) XgSolidLine;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[12]);
      XtManageChild(button);

  /** Point Size Selector Radio Box **/
  nargs = 0;
  XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNleftWidget, line_frame); ++nargs;
  XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  point_size_frame = XmCreateFrame(form, "point size frame", args, nargs);
  XtManageChild(point_size_frame);

  nargs = 0;
  button = XmCreateForm(point_size_frame, "point size form", args, nargs);
  XtManageChild(button);

  nargs = 0;
  frame_label = XmStringCreateLtoR("Point Size", XgCharset);
  XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_FORM); ++nargs;
  label = XmCreateLabel(button, "point size label", args, nargs);
  XtManageChild(label);
  XtFree(frame_label);

  nargs = 0;
  XtSetArg(args[nargs], XmNseparatorType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, label); ++nargs;
/*  XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;*/
  XtSetArg(args[nargs], XmNorientation, XmHORIZONTAL); ++nargs;
  separ = XmCreateSeparatorGadget(button, "Point Size Sep", args, nargs);
  XtManageChild(separ);

  nargs = 0;
  XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, separ); ++nargs;
  XtSetArg(args[nargs], XmNtopOffset, 5); ++nargs;
  radio = XmCreateRadioBox(button, "Point Size", args, nargs);
  XtManageChild(radio);
/*  XmAddTabGroup(radio);*/

      nargs = 0;
      if (graph->temp_data.point_size == XgSize3)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "3 Pixels", args, nargs);
      graph->wid_array[13].client = (caddr_t) XgSize3;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[13]);
      XtManageChild(button);
      nargs = 0;
      if (graph->temp_data.point_size == XgSize4)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "4 Pixels", args, nargs);
      graph->wid_array[14].client = (caddr_t) XgSize4;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[14]);
      XtManageChild(button);
      nargs = 0;
      if (graph->temp_data.point_size == XgSize5)
        { XtSetArg(args[nargs], XmNset, True); ++nargs; }
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "5 Pixels", args, nargs);
      graph->wid_array[15].client = (caddr_t) XgSize5;
      XtAddCallback(button, XmNarmCallback, Xg_toggle_data,
		    &graph->wid_array[15]);
      XtManageChild(button);

  /** Filled/Open, Left/Right, Error Bars On/Off Selector Radio Box **/
  nargs = 0;
  XtSetArg(args[nargs], XmNshadowType, XmSHADOW_ETCHED_IN); ++nargs;
  XtSetArg(args[nargs], XmNleftAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNleftWidget, point_type_frame); ++nargs;
  XtSetArg(args[nargs], XmNleftOffset, 10); ++nargs;
  XtSetArg(args[nargs], XmNrightAttachment, XmATTACH_FORM); ++nargs;
  XtSetArg(args[nargs], XmNtopAttachment, XmATTACH_WIDGET); ++nargs;
  XtSetArg(args[nargs], XmNtopWidget, point_size_frame); ++nargs;
  XtSetArg(args[nargs], XmNtopOffset, 10); ++nargs;
  choices_frame = XmCreateFrame(form, "choices frame", args, nargs);
  XtManageChild(choices_frame);

  nargs = 0;
  XtSetArg(args[nargs], XmNentryClass, xmToggleButtonGadgetClass); ++nargs;
  XtSetArg(args[nargs], XmNisHomogeneous, True); ++nargs;
  XtSetArg(args[nargs], XmNindicatorType, XmN_OF_MANY); ++nargs;
  radio = XmCreateRowColumn(choices_frame, "Choices Radio", args, nargs);
  XtManageChild(radio);
/*  XmAddTabGroup(radio);*/

      /**** Filled/Open Button ****/
      nargs = 0;
      if (graph->temp_data.filled_open == XgFilled)
        {
	  XtSetArg(args[nargs], XmNset, True); ++nargs;
	  frame_label = XmStringCreateLtoR("Pts FILLED/open", XgCharset);
	}
      else
	{
	  XtSetArg(args[nargs], XmNset, False); ++nargs;
	  frame_label = XmStringCreateLtoR("Pts filled/OPEN", XgCharset);
	}
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Pts Filled/Open button",
					  args, nargs);
      graph->wid_array[16].client = (caddr_t) XgFilled;
      XtAddCallback(button, XmNvalueChangedCallback, Xg_toggle_data,
		    &graph->wid_array[16]);
      XtManageChild(button);
      XtFree(frame_label);

      /**** Left/Right Button ****/
      nargs = 0;
      if (graph->temp_data.left_right == XgLeft)
        {
	  XtSetArg(args[nargs], XmNset, True); ++nargs;
	  frame_label = XmStringCreateLtoR("LEFT/right Axis", XgCharset);
	}
      else
        {
	  XtSetArg(args[nargs], XmNset, False); ++nargs;
	  frame_label = XmStringCreateLtoR("left/RIGHT Axis", XgCharset);
	}
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Left/Right button",
					  args, nargs);
      graph->wid_array[18].client = (caddr_t) XgLeft;
      XtAddCallback(button, XmNvalueChangedCallback, Xg_toggle_data,
		    &graph->wid_array[18]);
      nargs = 0;  /** don't allow last dataset to be set to right axis **/
      for (i=0; i<graph->num_sets; ++i)
	if (graph->dataset[i].left_right == XgLeft)
	  ++nargs;
      if ((nargs <= 1) && (graph->temp_data.left_right == XgLeft))
	XtSetSensitive(button, False);
      XtManageChild(button);
      XtFree(frame_label);

      /***** error bars on/off *********/
      nargs = 0;
      if (graph->temp_data.error_bars == XgErrorOn)
	{
	  XtSetArg(args[nargs], XmNset, True); ++nargs;
	  frame_label = XmStringCreateLtoR("Error Bars ON/off", XgCharset);
	}
      else
	{
	  XtSetArg(args[nargs], XmNset, True); ++nargs;
	  frame_label = XmStringCreateLtoR("Error Bars on/OFF", XgCharset);
	}
      XtSetArg(args[nargs], XmNshadowThickness, 0); ++nargs;
      XtSetArg(args[nargs], XmNlabelString, frame_label); ++nargs;
      button = XmCreateToggleButtonGadget(radio, "Error Bars On/Off",
					  args, nargs);
      graph->wid_array[20].client = (caddr_t) XgErrorOn;
      XtAddCallback(button, XmNvalueChangedCallback, Xg_toggle_data,
		    &graph->wid_array[20]);
      if (graph->temp_data.error_pts != XgHasErrorPts)
	XtSetSensitive(button, False);
      XtManageChild(button);
      XtFree(frame_label);

/**** add main popup callbacks ****/
XtDestroyWidget(w);
XtManageChild(parent);
XtAddCallback(parent, XmNcancelCallback, Xg_cancel_CB, NULL);
XtAddCallback(parent, XmNhelpCallback, Xg_help_sys,
	      (caddr_t) XgHELP_DATA_OPTIONS);
XtAddCallback(parent, XmNokCallback, Xg_set_data_vals, client_data);
XtAddCallback(parent, XmNapplyCallback, Xg_set_data_defaults, client_data);
XtFree(title);
XtFree(defaults);

}

/****************************/
void Xg_get_temp_legend(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    *buf;
  int
    i;
  double
    value;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
buf = XmTextGetString(w);
strcpy(graph->temp_data.legend_text, buf);
XtFree(buf);
}

/**************************************/
void Xg_toggle_data(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    val,
    nargs;
  Arg
    args[5];
  XmString
    label;

  XgGraphInfo
    *graph;
  XgWidStruct
    *wid_addr;
  XmToggleButtonCallbackStruct
    *toggle_info;

toggle_info = (XmToggleButtonCallbackStruct *) call_data;
wid_addr = (XgWidStruct *) client_data;
graph = (XgGraphInfo *) wid_addr->graph;
val = (int) wid_addr->client;

switch (val)
  {
  case XgInLegend:
    if (toggle_info->set == True)
      {
	graph->temp_data.in_legend = XgInLegend;
	label = XmStringCreateLtoR("IN/out of Legend", XgCharset);
      }
    else if (toggle_info->set == False)
      {
	graph->temp_data.in_legend = XgNotInLegend;
	label = XmStringCreateLtoR("in/OUT of Legend", XgCharset);
      }
    else
      fprintf(stderr, "ERROR in toggle_info->set value\n");
    nargs = 0;
    XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
    XtSetValues(w, args, nargs);
    XtFree(label);
    break;

  case XgPoint:
  case XgCircle:
  case XgSquare:
  case XgTriangle:
  case XgPlus:
  case XgEx:
  case XgBar:
    graph->temp_data.point_type = val;
    break;

  case XgNoLine:
  case XgSolidLine:
    graph->temp_data.line_status = val;
    break;

  case XgSize3:
  case XgSize4:
  case XgSize5:
    graph->temp_data.point_size = val;
    break;

  case XgFilled:   /** toggle_info->set contains value BEFORE button pushed **/
    if (toggle_info->set == True)
      {
	graph->temp_data.filled_open = XgFilled;
	label = XmStringCreateLtoR("Pts FILLED/open", XgCharset);
      }
    else if (toggle_info->set == False)
      {
	graph->temp_data.filled_open = XgOpen;
	label = XmStringCreateLtoR("Pts filled/OPEN", XgCharset);
      }
    else
      fprintf(stderr, "ERROR in toggle_info->set value\n");
    nargs = 0;
    XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
    XtSetValues(w, args, nargs);
    XtFree(label);
    break;

  case XgLeft:
    if (toggle_info->set == True)
      {
	graph->temp_data.left_right = XgLeft;
	label = XmStringCreateLtoR("LEFT/right Axis", XgCharset);
      }
    else if (toggle_info->set == False)
      {
	graph->temp_data.left_right = XgRight;
	label = XmStringCreateLtoR("left/RIGHT Axis", XgCharset);
      }
    else
      fprintf(stderr, "ERROR in toggle_info->set value\n");
    nargs = 0;
    XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
    XtSetValues(w, args, nargs);
    XtFree(label);
    break;

  case XgErrorOn:
    if (toggle_info->set == True)
      {
	graph->temp_data.error_bars = XgErrorOn;
	label = XmStringCreateLtoR("Error Bars ON/off", XgCharset);
      }
    else if (toggle_info->set == False)
      {
	graph->temp_data.error_bars = XgErrorOff;
	label = XmStringCreateLtoR("Error Bars on/OFF", XgCharset);
      }
    else
      fprintf(stderr, "ERROR in toggle_info->set value\n");
    nargs = 0;
    XtSetArg(args[nargs], XmNlabelString, label); ++nargs;
    XtSetValues(w, args, nargs);
    XtFree(label);
    break;

  default:
    fprintf(stderr, "ERROR: bad client type = %d in Xg_toggle_data\n", val);
    break;
  }
}

/**************************************/
void Xg_set_data_vals(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
i = graph->temp_data.num_points; /* this was holding the selected data set # */

if ((graph->temp_coord.x < 0.) || (graph->temp_coord.x > 100.))
  {
    Xg_error_sys(XgERR_BAD_COORDS, graph);
    return;
  }
if ((graph->temp_coord.y < 0.) || (graph->temp_coord.y > 100.))
  {
    Xg_error_sys(XgERR_BAD_COORDS, graph);
    return;
  }

strcpy(graph->dataset[i].description.text, graph->temp_text);
if (strcmp(graph->dataset[i].description.font_info.name, graph->temp_font))
  {   /** new font **/
    XtFree(graph->dataset[i].description.font_info.font_struct);
    graph->dataset[i].description.font_info.font_struct
      = XLoadQueryFont(graph->display, graph->temp_font);
    if (graph->dataset[i].description.font_info.font_struct == NULL)
      {
	graph->dataset[i].description.font_info.font_struct
	  = XLoadQueryFont(graph->display,
			   graph->dataset[i].description.font_info.name);
	Xg_error_sys(XgERR_NO_FONT, graph);
      }
    else
      strcpy(graph->dataset[i].description.font_info.name, graph->temp_font);
  }
graph->dataset[i].description.coord.x = graph->temp_coord.x;
graph->dataset[i].description.coord.y = graph->temp_coord.y;

strcpy(graph->dataset[i].legend_text, graph->temp_data.legend_text);
graph->dataset[i].in_legend = graph->temp_data.in_legend;

graph->dataset[i].point_type = graph->temp_data.point_type;
graph->dataset[i].point_size = graph->temp_data.point_size;
graph->dataset[i].filled_open = graph->temp_data.filled_open;
graph->dataset[i].line_status = graph->temp_data.line_status;
graph->dataset[i].left_right = graph->temp_data.left_right;
graph->dataset[i].error_bars = graph->temp_data.error_bars;

XtDestroyWidget(w);
}

/**************************************/
void Xg_set_data_defaults(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
i = graph->temp_data.num_points;
XtDestroyWidget(w);

graph->dataset[i].error_bars = XgDefErrorBarStatus;
graph->dataset[i].point_type = XgDefPointType;
graph->dataset[i].point_size = XgDefPtSize;
graph->dataset[i].filled_open = XgDefFillOpen;
graph->dataset[i].line_status = XgDefLineType;
graph->dataset[i].in_legend = XgDefLegendStatus;
strcpy(graph->dataset[i].legend_text, graph->dataset[i].filename);
graph->dataset[i].left_right = XgLeft;
graph->dataset[i].description.text[0] = '\0';
graph->dataset[i].description.coord.x = XgDefDataDescXCoord;
graph->dataset[i].description.coord.y = XgDefDataDescYCoord;
graph->dataset[i].description.dimen.x = 0;
graph->dataset[i].description.dimen.y = 0;
graph->dataset[i].description.font_info.font_struct
  = XLoadQueryFont(graph->display, XgDefFont);
strcpy(graph->dataset[i].description.font_info.name, XgDefFont);

/*Xg_drawGraph(graph);*/
}
