/**********************************************************************
*
*                            XgDrawAxes.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*  Draw Graph Axes w/ scaling
*
***********************************************************************
*/

#include "stdio.h"
#include "math.h"
#include "string.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

/**********************************************************************/
void Xg_drawAxes(XgGraphInfo * graph)
{
  int
    i,
    j,
    width,
    height,
    de,
    exp;
  double
    x,
    y,
    z,
    x1,
    y1,
    x0,
    dx,
    min,
    max,
    inc;
  DEV_COORD
    nx,
    ny;
  char
    *chrptr,
    buf[80],
    buf2[20],
    format[10];

XSetFont(graph->display, graph->theGC,
	 graph->legend.description.font_info.font_struct->fid);

/***** x-axis numbering *******/
graph->xaxis_layout.scale_desc.dimen.x = 0;
graph->xaxis_layout.scale_desc.dimen.y = 0;

if (graph->xaxis_layout.scaling == XgLogarithm)
  {
    y = graph->yaxis_layout.sclmin;
    y1 = graph->yaxis_layout.sclmax;
    min = graph->xaxis_layout.sclmin;
    max = graph->xaxis_layout.sclmax;
    for (x=min; x<=max; ++x)
      {
	Xg_Mapwtod(x, y, &nx, &ny, graph, XgLeft); /* tick marks across btm */
	XDrawLine(graph->display, XtWindow(graph->draw_area),
		  graph->theGC, nx, ny, nx, ny + 5);
	z = x;
	i = x;
	sprintf(buf2, "10^%d", i);
	if ((max <= 5) && (min >= -4))
	  {
	    z = pow(10., x);
	    sprintf(buf2, "%lg", z);
	  }
	ny += 5 + graph->legend.description.font_info.font_struct->ascent
	  + graph->legend.description.font_info.font_struct->descent;
	nx -= XTextWidth(graph->legend.description.font_info.font_struct, buf2,
			 strlen(buf2)) / 2; /* x coord is center of string */
	XDrawString(graph->display, XtWindow(graph->draw_area),
		    graph->theGC, nx, ny, buf2, strlen(buf2));
      }
  }  /** end x-axis Logarithm **/

if (graph->xaxis_layout.scaling == XgLinear)
  {
    /** get exp **/
    max = fabs(graph->xaxis_layout.sclmax);
    min = fabs(graph->xaxis_layout.sclmin);
    if (max > min)
      sprintf(buf, "%le", 0.9999 * max);
    else  /** the .9999 avoids a problem if max is an exact power of 10 **/
      sprintf(buf, "%le", 0.9999 * min);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%d", &exp);

    de = 0;
    if (graph->xaxis_layout.label_type == XgEng_label)
      {
	while (exp/3. != floor(exp/3.))
	  {
	    --exp;
	    ++de;
	  }
      }

    if ( (graph->xaxis_layout.label_type == XgSci_label)
	|| (graph->xaxis_layout.label_type == XgEng_label) )
      {
	sprintf(buf, "x data = graph * 10^%d", exp);
	nx = graph->xaxis_layout.scale_desc.coord.x * graph->nxdraw / 100.;
	ny = graph->xaxis_layout.scale_desc.coord.y * graph->nydraw / 100.;
	width = XTextWidth(graph->legend.description.font_info.font_struct,
			   buf, strlen(buf));
	if (nx + width + 5 > graph->nxdraw)
	  {
	    nx = graph->nxdraw - width - 5;
	    graph->xaxis_layout.scale_desc.coord.x = nx * 100. / graph->nxdraw;
	  }
	height = graph->legend.description.font_info.font_struct->ascent
	  + graph->legend.description.font_info.font_struct->descent;
	ny += height;
	if (exp != 0)
	  {
	    graph->xaxis_layout.scale_desc.dimen.x = width;
	    graph->xaxis_layout.scale_desc.dimen.y = height;
	    XDrawString(graph->display, XtWindow(graph->draw_area),
			graph->theGC, nx, ny, buf, strlen(buf));
	  }

	j = graph->xaxis_layout.nsig_fig - XgSig0 - 1 - de;
	if (j < 0)
	  j = 0;
	sprintf(format, "%%%d.%dlf",
		graph->xaxis_layout.nsig_fig - XgSig0 + 1, j);
      }  /** end scientific notation label **/

    if (graph->xaxis_layout.label_type == XgNo_label)
      {
	sprintf(format, "%%lg");
	if ((exp == 0) && (graph->xaxis_layout.nsig_fig == XgSig2))
	  sprintf(format, "%%3.1lf");  /** e.g. 8.9, 9.0 **/
	if ((exp == 0) && (graph->xaxis_layout.nsig_fig == XgSig3))
	  sprintf(format, "%%4.2lf");  /** e.g. 8.95, 9.00 **/
	if ((exp == 1) && (graph->xaxis_layout.nsig_fig == XgSig3))
	  sprintf(format, "%%4.1lf");  /** e.g. 89.5, 90.0 **/
	if (exp < 0)   /** e.g. 0.055, 0.060 **/
	  {
	    i = -exp + graph->xaxis_layout.nsig_fig - XgSig0 - 1;
	    sprintf(format, "%%%d.%dlf", i+2, i);
	  }
      }  /** end of no scientific notation labels **/

    y = graph->yaxis_layout.sclmin;
    y1 = graph->yaxis_layout.sclmax;
    min = graph->xaxis_layout.sclmin;
    max = graph->xaxis_layout.sclmax;
    inc = pow(10., (double) ((exp + de)
			     - (graph->xaxis_layout.nsig_fig - XgSig0-1)));
    inc = (double) (graph->xaxis_layout.increment - XgLast0) * inc;

    for (x=min; x<=1.0005*max; x+=inc)
      {
	Xg_Mapwtod(x, y, &nx, &ny, graph, XgLeft); /* tick marks across btm */
	XDrawLine(graph->display, XtWindow(graph->draw_area),
		  graph->theGC, nx, ny, nx, ny + 5);
	z = x;
	if ( (graph->xaxis_layout.label_type == XgSci_label)
	    || (graph->xaxis_layout.label_type == XgEng_label) )
	  z = z / pow(10., (double) exp);
	else
	  if (fabs(z) < inc / 10.)
	    z = 0.;
	sprintf(buf, format, z);
	sscanf(buf, "%s", buf2);
	ny += 5 + graph->legend.description.font_info.font_struct->ascent
	  + graph->legend.description.font_info.font_struct->descent;
	nx -= XTextWidth(graph->legend.description.font_info.font_struct, buf2,
			 strlen(buf2)) / 2; /* x coord is center of string */
	XDrawString(graph->display, XtWindow(graph->draw_area),
		    graph->theGC, nx, ny, buf2, strlen(buf2));
      }  /** end of loop over labels **/

  }  /** end x-axis Linear **/

/***** y-axis numbering *******/
graph->yaxis_layout.scale_desc.dimen.x = 0;
graph->yaxis_layout.scale_desc.dimen.y = 0;

if (graph->yaxis_layout.scaling == XgLogarithm)
  {
    x = graph->xaxis_layout.sclmin;
    x1 = graph->xaxis_layout.sclmax;
    min = graph->yaxis_layout.sclmin;
    max = graph->yaxis_layout.sclmax;
    for (y=min; y<=max; ++y)
      {
	Xg_Mapwtod(x, y, &nx, &ny, graph, XgLeft); /* tick marks on left side*/
	XDrawLine(graph->display, XtWindow(graph->draw_area),
		  graph->theGC, nx-1, ny, nx - 5, ny);
	z = y;
	i = y;
	sprintf(buf2, "10^%d", i);
	if ((max <= 5) && (min >= -4))
	  {
	    z = pow(10., y);
	    sprintf(buf2, "%lg", z);
	  }
	ny += (graph->legend.description.font_info.font_struct->ascent) / 2;
/*	   + graph->legend.description.font_info.font_struct->descent) / 2;*/
	nx -= (5 + XTextWidth(graph->legend.description.font_info.font_struct,
			      buf2, strlen(buf2)));
	XDrawString(graph->display, XtWindow(graph->draw_area),
		    graph->theGC, nx, ny, buf2, strlen(buf2));
      }
  }  /** end y-axis Logarithm **/

if (graph->yaxis_layout.scaling == XgProbability)
  {
    x = graph->xaxis_layout.sclmin;
    x1 = graph->xaxis_layout.sclmax;
    min = graph->yaxis_layout.prob_mindx;
    max = graph->yaxis_layout.prob_maxdx;
    for (i=min; i<=max; ++i)
      {
	y1 = Xg_inverf(XgProb_array[i]);
	Xg_Mapwtod(x, y1, &nx, &ny, graph, XgLeft); /*tick marks on left side*/
	XDrawLine(graph->display, XtWindow(graph->draw_area),
		  graph->theGC, nx-1, ny, nx - 5, ny);
	sprintf(buf2, "%lg", XgProb_array[i]);
	ny += (graph->legend.description.font_info.font_struct->ascent) / 2;
/*	   + graph->legend.description.font_info.font_struct->descent) / 2;*/
	nx -= (5 + XTextWidth(graph->legend.description.font_info.font_struct,
			      buf2, strlen(buf2)));
	XDrawString(graph->display, XtWindow(graph->draw_area),
		    graph->theGC, nx, ny, buf2, strlen(buf2));
      }
  }  /** end y-axis Probability **/

if (graph->yaxis_layout.scaling == XgLinear)
  {
    /** get exp **/
    max = fabs(graph->yaxis_layout.sclmax);
    min = fabs(graph->yaxis_layout.sclmin);
    if (max > min)
      sprintf(buf, "%le", 0.9999 * max);
    else  /** the .9999 avoids a problem if max is an exact power of 10 **/
      sprintf(buf, "%le", 0.9999 * min);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%d", &exp);

    de = 0;
    if (graph->yaxis_layout.label_type == XgEng_label)
      {
	while(exp/3. != floor(exp/3.))
	  {
	    --exp;
	    ++de;
	  }
      }

    if ( (graph->yaxis_layout.label_type == XgSci_label)
	|| (graph->yaxis_layout.label_type == XgEng_label) )
      {
	sprintf(buf, "y data = graph * 10^%d", exp);
	nx = graph->yaxis_layout.scale_desc.coord.x * graph->nxdraw / 100.;
	ny = graph->yaxis_layout.scale_desc.coord.y * graph->nydraw / 100.;
	width = XTextWidth(graph->legend.description.font_info.font_struct,
			   buf, strlen(buf));
	if (nx + width + 5 > graph->nxdraw)
	  {
	    nx = graph->nxdraw - width - 5;
	    graph->yaxis_layout.scale_desc.coord.x = nx * 100. / graph->nxdraw;
	  }
	height = graph->legend.description.font_info.font_struct->ascent
	  + graph->legend.description.font_info.font_struct->descent;
	ny += height;
	if (exp != 0)
	  {
	    graph->yaxis_layout.scale_desc.dimen.x = width;
	    graph->yaxis_layout.scale_desc.dimen.y = height;
	    XDrawString(graph->display, XtWindow(graph->draw_area),
			graph->theGC, nx, ny, buf, strlen(buf));
	  }

	j = graph->yaxis_layout.nsig_fig - XgSig0 - 1 - de;
	if (j < 0)
	  j = 0;
	sprintf(format, "%%%d.%dlf",
		graph->yaxis_layout.nsig_fig - XgSig0 + 1, j);
      }  /** end sci. not. label **/

    if (graph->yaxis_layout.label_type == XgNo_label)
      {
	sprintf(format, "%%lg");
	if ((exp == 0) && (graph->yaxis_layout.nsig_fig == XgSig2))
	  sprintf(format, "%%3.1lf");  /** e.g. 8.9, 9.0 **/
	if ((exp == 0) && (graph->yaxis_layout.nsig_fig == XgSig3))
	  sprintf(format, "%%4.2lf");  /** e.g. 8.95, 9.00 **/
	if ((exp == 1) && (graph->yaxis_layout.nsig_fig == XgSig3))
	  sprintf(format, "%%4.1lf");  /** e.g. 89.5, 90.0 **/
	if (exp < 0)   /** e.g. 0.055, 0.060 **/
	  {
	    i = -exp + graph->yaxis_layout.nsig_fig - XgSig0 - 1;
	    sprintf(format, "%%%d.%dlf", i+2, i);
	  }
      }  /** end of no scientific notation labels **/

    x = graph->xaxis_layout.sclmin;
    x1= graph->xaxis_layout.sclmax;
    min = graph->yaxis_layout.sclmin;
    max = graph->yaxis_layout.sclmax;
    inc = pow(10., (double) ((exp + de)
			     - (graph->yaxis_layout.nsig_fig - XgSig0-1)));
    inc = (double) (graph->yaxis_layout.increment - XgLast0) * inc;

    for (y=min; y<=1.0005*max; y+=inc)
      {
	Xg_Mapwtod(x, y, &nx, &ny, graph, XgLeft); /* tick marks on left side*/
	XDrawLine(graph->display, XtWindow(graph->draw_area),
		  graph->theGC, nx-1, ny, nx - 5, ny);
	z = y;
	if ( (graph->yaxis_layout.label_type == XgSci_label)
	    || (graph->yaxis_layout.label_type == XgEng_label) )
	  z = z / pow(10., (double) exp);
	else
	  if (fabs(z) < inc / 10.)
	    z = 0.;
	sprintf(buf, format, z);
	sscanf(buf, "%s", buf2);
	ny += (graph->legend.description.font_info.font_struct->ascent) / 2;
/*	   + graph->legend.description.font_info.font_struct->descent) / 2;*/
	nx -= (5 + XTextWidth(graph->legend.description.font_info.font_struct,
			      buf2, strlen(buf2)));
	XDrawString(graph->display, XtWindow(graph->draw_area),
		    graph->theGC, nx, ny, buf2, strlen(buf2));
      }
  }  /** end y-axis Linear **/

/********** Right y-axis numbering *************/
if (graph->use_right == TRUE)
  {
    graph->y_rt_axis_layout.scale_desc.dimen.x = 0;
    graph->y_rt_axis_layout.scale_desc.dimen.y = 0;

    if (graph->y_rt_axis_layout.scaling == XgLogarithm)
      {
	x = graph->xaxis_layout.sclmin;
	x1 = graph->xaxis_layout.sclmax;
	min = graph->y_rt_axis_layout.sclmin;
	max = graph->y_rt_axis_layout.sclmax;
	for (y=min; y<=max; ++y)
	  {
	    /* tick marks on right side*/
	    Xg_Mapwtod(x1, y, &nx, &ny, graph, XgRight);
	    XDrawLine(graph->display, XtWindow(graph->draw_area),
		      graph->theGC, nx+1, ny, nx + 5, ny);
	    z = y;
	    i = y;
	    sprintf(buf2, "10^%d", i);
	    if ((max <= 5) && (min >= -4))
	      {
		z = pow(10., y);
		sprintf(buf2, "%lg", z);
	      }
	    ny += (graph->legend.description.font_info.font_struct->ascent) /2;
/*	      + graph->legend.description.font_info.font_struct->descent) /2;*/
	    nx += 8;
	    XDrawString(graph->display, XtWindow(graph->draw_area),
			graph->theGC, nx, ny, buf2, strlen(buf2));
	  }
      }  /** end y-axis Logarithm **/

    if (graph->y_rt_axis_layout.scaling == XgProbability)
      {
	x = graph->xaxis_layout.sclmin;
	x1 = graph->xaxis_layout.sclmax;
	min = graph->y_rt_axis_layout.prob_mindx;
	max = graph->y_rt_axis_layout.prob_maxdx;
	for (i=min; i<=max; ++i)
	  {
	    /*tick marks on right side*/
	    y1 = Xg_inverf(XgProb_array[i]);
	    Xg_Mapwtod(x1, y1, &nx, &ny, graph, XgRight);
	    XDrawLine(graph->display, XtWindow(graph->draw_area),
		      graph->theGC, nx+1, ny, nx + 5, ny);
	    sprintf(buf2, "%lg", XgProb_array[i]);
	    ny += (graph->legend.description.font_info.font_struct->ascent) /2;
/*	      + graph->legend.description.font_info.font_struct->descent) / 2*/
	    nx += 8;
	    XDrawString(graph->display, XtWindow(graph->draw_area),
			graph->theGC, nx, ny, buf2, strlen(buf2));
	  }
      }  /** end y-axis Probability **/

    if (graph->y_rt_axis_layout.scaling == XgLinear)
      {
	/** get exp **/
	max = fabs(graph->y_rt_axis_layout.sclmax);
	min = fabs(graph->y_rt_axis_layout.sclmin);
	if (max > min)
	  sprintf(buf, "%le", 0.9999 * max);
	else  /** the .9999 avoids a problem if max is an exact power of 10 **/
	  sprintf(buf, "%le", 0.9999 * min);
	chrptr = strchr(buf, 'e');
	++chrptr;
	if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
	  ++chrptr;
	sscanf(chrptr, "%d", &exp);

	de = 0;
	if (graph->y_rt_axis_layout.label_type == XgEng_label)
	  {
	    while(exp/3. != floor(exp/3.))
	      {
		--exp;
		++de;
	      }
	  }

	if ( (graph->y_rt_axis_layout.label_type == XgSci_label)
	    || (graph->y_rt_axis_layout.label_type == XgEng_label) )
	  {
	    sprintf(buf, "yr data = graph * 10^%d", exp);
	    nx = graph->y_rt_axis_layout.scale_desc.coord.x*graph->nxdraw/100.;
	    ny = graph->y_rt_axis_layout.scale_desc.coord.y*graph->nydraw/100.;
	    width = XTextWidth(graph->legend.description.font_info.font_struct,
			       buf, strlen(buf));
	    if (nx + width + 5 > graph->nxdraw)
	      {
		nx = graph->nxdraw - width - 5;
		graph->y_rt_axis_layout.scale_desc.coord.x
		  = nx * 100. / graph->nxdraw;
	      }
	    height = graph->legend.description.font_info.font_struct->ascent
	      + graph->legend.description.font_info.font_struct->descent;
	    ny += height;
	    if (exp != 0)
	      {
		graph->y_rt_axis_layout.scale_desc.dimen.x = width;
		graph->y_rt_axis_layout.scale_desc.dimen.y = height;
		XDrawString(graph->display, XtWindow(graph->draw_area),
			    graph->theGC, nx, ny, buf, strlen(buf));
	      }

	    j = graph->y_rt_axis_layout.nsig_fig - XgSig0 - 1 - de;
	    if (j < 0)
	      j = 0;
	    sprintf(format, "%%%d.%dlf",
		    graph->y_rt_axis_layout.nsig_fig - XgSig0 + 1, j);
	  }  /** end sci. not. label **/

	if (graph->y_rt_axis_layout.label_type == XgNo_label)
	  {
	    sprintf(format, "%%lg");
	    if ((exp == 0) && (graph->y_rt_axis_layout.nsig_fig == XgSig2))
	      sprintf(format, "%%3.1lf");  /** e.g. 8.9, 9.0 **/
	    if ((exp == 0) && (graph->y_rt_axis_layout.nsig_fig == XgSig3))
	      sprintf(format, "%%4.2lf");  /** e.g. 8.95, 9.00 **/
	    if ((exp == 1) && (graph->y_rt_axis_layout.nsig_fig == XgSig3))
	      sprintf(format, "%%4.1lf");  /** e.g. 89.5, 90.0 **/
	    if (exp < 0)   /** e.g. 0.055, 0.060 **/
	      {
		i = -exp + graph->y_rt_axis_layout.nsig_fig - XgSig0 - 1;
		sprintf(format, "%%%d.%dlf", i+2, i);
	      }
	  }  /** end of no scientific notation labels **/

	x = graph->xaxis_layout.sclmin;
	x1= graph->xaxis_layout.sclmax;
	min = graph->y_rt_axis_layout.sclmin;
	max = graph->y_rt_axis_layout.sclmax;
	inc = pow(10., (double) ((exp + de)
			    - (graph->y_rt_axis_layout.nsig_fig - XgSig0-1)));
	inc = (double) (graph->y_rt_axis_layout.increment - XgLast0) * inc;

	for (y=min; y<=1.0005*max; y+=inc)
	  {
	    /* tick marks on right side*/
	    Xg_Mapwtod(x1, y, &nx, &ny, graph, XgRight);
	    XDrawLine(graph->display, XtWindow(graph->draw_area),
		      graph->theGC, nx+1, ny, nx + 5, ny);
	    z = y;
	    if ( (graph->y_rt_axis_layout.label_type == XgSci_label)
		|| (graph->y_rt_axis_layout.label_type == XgEng_label) )
	      z = z / pow(10., (double) exp);
	    else
	      if (fabs(z) < inc / 10.)
		z = 0.;
	    sprintf(buf, format, z);
	    sscanf(buf, "%s", buf2);
	    ny += (graph->legend.description.font_info.font_struct->ascent) /2;
/*	      + graph->legend.description.font_info.font_struct->descent) /2;*/
	    nx += 8;
	    XDrawString(graph->display, XtWindow(graph->draw_area),
			graph->theGC, nx, ny, buf2, strlen(buf2));
	  }
      }  /** end y-axis Linear **/
  }  /** end of y-rt-axis **/

}
