/**********************************************************************
*
*                            XgDrawDivs.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*  Draw Graph Division Marks (Tics & Lines)
*
***********************************************************************
*/

#include "stdio.h"
#include "math.h"
#include "string.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

/**********************************************************************/
void Xg_drawDivMarks(XgGraphInfo * graph)
{
  int
    i,
    j,
    de,
    exp;
  double
    x,
    y,
    z,
    x1,
    y1,
    dx,
    min,
    max,
    inc;
  char
    *chrptr,
    buf[80];
  DEV_COORD
    nx,
    ny,
    nx1,
    ny1;
  XGCValues
    gcvalues;

/******************* x axis div marks *********************/
if (graph->xaxis_layout.scaling == XgLogarithm)
  {
    inc = 1.;
  }
else if (graph->xaxis_layout.scaling == XgLinear)
  {
    /** get exp **/
    max = fabs(graph->xaxis_layout.sclmax);
    min = fabs(graph->xaxis_layout.sclmin);
    if (max > min)
      sprintf(buf, "%le", 0.9999 * max);
    else  /** the .9999 avoids a problem if max is an exact power of 10 **/
      sprintf(buf, "%le", 0.9999 * min);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    sscanf(chrptr, "%d", &exp);

    de = 0;
    if (graph->xaxis_layout.label_type == XgEng_label)
      {
	while (exp/3. != floor(exp/3.))
	  {
	    --exp;
	    ++de;
	  }
      }

    inc = pow(10., (double) ((exp + de)
			     - (graph->xaxis_layout.nsig_fig - XgSig0-1)));
    inc = (double) (graph->xaxis_layout.increment - XgLast0) * inc;
  }

y = graph->yaxis_layout.sclmin;
y1 = graph->yaxis_layout.sclmax;
min = graph->xaxis_layout.sclmin;
max = graph->xaxis_layout.sclmax;

for (x=min; x<max; x+=inc)
  {
    if ( ( (graph->xaxis_layout.div_marks == XgDivMajorTic)
	  || (graph->xaxis_layout.div_marks == XgDivMinorTic) )
	&& (x>min) && (x<max))
      {
	/** tick marks across bottom **/
	Xg_Mapwtod(x, y, &nx, &ny, graph, XgLeft);
	XDrawLine(graph->display, XtWindow(graph->draw_area),
		  graph->theGC, nx, ny, nx, ny - 10);
	/** tick marks across top **/
	Xg_Mapwtod(x, y1, &nx, &ny, graph, XgLeft);
	XDrawLine(graph->display, XtWindow(graph->draw_area),
		  graph->theGC, nx, ny, nx, ny + 10);
      }  /** end of Major Div Tics **/

    if ( ( (graph->xaxis_layout.div_marks == XgDivMajorLin)
	  || (graph->xaxis_layout.div_marks == XgDivMinorLin)
	  || (graph->xaxis_layout.div_marks == XgDivLin_Tic) )
	&& (x>min) && (x<max))
      {
/*	gcvalues.line_style = LineOnOffDash;
	XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);*/
	Xg_Mapwtod(x, y, &nx, &ny, graph, XgLeft);
	Xg_Mapwtod(x, y1, &nx1, &ny1, graph, XgLeft);
	XDrawLine(graph->display, XtWindow(graph->draw_area),
		  graph->theGC, nx, ny, nx1, ny1);
/*	gcvalues.line_style = LineSolid;
	XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);*/
      }  /** end of Major Div Lines **/

    if ((graph->xaxis_layout.div_marks == XgDivMinorTic)
	  || (graph->xaxis_layout.div_marks == XgDivLin_Tic))
      {
	if (graph->xaxis_layout.scaling == XgLogarithm)
	  {
	    dx = pow(10., x);
	    for (i=2; i<10; ++i)
	      {
		z = log10(i*dx);
		Xg_Mapwtod(z, y, &nx, &ny, graph, XgLeft);
		XDrawLine(graph->display, XtWindow(graph->draw_area),
			  graph->theGC, nx, ny, nx, ny - 5);
		Xg_Mapwtod(z, y1, &nx, &ny, graph, XgLeft);
		XDrawLine(graph->display, XtWindow(graph->draw_area),
			  graph->theGC, nx, ny, nx, ny + 5);
	      }
	  }  /** end of XgLogarithm **/
	else if (graph->xaxis_layout.scaling == XgLinear)
	  {
	    if (graph->xaxis_layout.increment == XgLast2)
	      j = 2;
	    else if (graph->xaxis_layout.increment == XgLast5)
	      j = 5;
	    else if (graph->xaxis_layout.increment == XgLast1)
	      j = 10;
	    dx = inc / (double) j;
	    for (i=1; i<j; ++i)
	      {
		z = x + dx*i;
		if ((z > min) && (z < max))
		  {
		    Xg_Mapwtod(z, y, &nx, &ny, graph, XgLeft);
		    XDrawLine(graph->display, XtWindow(graph->draw_area),
			      graph->theGC, nx, ny, nx, ny - 5);
		    Xg_Mapwtod(z, y1, &nx, &ny, graph, XgLeft);
		    XDrawLine(graph->display, XtWindow(graph->draw_area),
			      graph->theGC, nx, ny, nx, ny + 5);
		  }
	      }
	  }  /** end of Linear **/
      }  /** end of Minor Div Tics **/

    if (graph->xaxis_layout.div_marks == XgDivMinorLin)
      {
	gcvalues.line_style = LineOnOffDash;
	XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);
	if (graph->xaxis_layout.scaling == XgLogarithm)
	  {
	    dx = pow(10., x);
	    for (i=2; i<10; ++i)
	      {
		z = log10(i*dx);
		Xg_Mapwtod(z, y, &nx, &ny, graph, XgLeft);
		Xg_Mapwtod(z, y1, &nx1, &ny1, graph, XgLeft);
		XDrawLine(graph->display, XtWindow(graph->draw_area),
			  graph->theGC, nx, ny, nx1, ny1);
	      }
	  }
	else if (graph->xaxis_layout.scaling == XgLinear)
	  {
	    if (graph->xaxis_layout.increment == XgLast2)
	      j = 2;
	    else if (graph->xaxis_layout.increment == XgLast5)
	      j = 5;
	    else if (graph->xaxis_layout.increment == XgLast1)
	      j = 10;
	    dx = inc / (double) j;
	    for (i=1; i<j; ++i)
	      {
		z = x + dx*i;
		if ((z > min) && (z < max))
		  {
		    Xg_Mapwtod(z, y, &nx, &ny, graph, XgLeft);
		    Xg_Mapwtod(z, y1, &nx1, &ny1, graph, XgLeft);
		    XDrawLine(graph->display, XtWindow(graph->draw_area),
			      graph->theGC, nx, ny, nx1, ny1);
		  }
	      }
	  }  /** end of Linear **/
	gcvalues.line_style = LineSolid;
	XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);
      }  /** end of Minor Div Lines **/
  }  /** end of x loop **/

/******************* y axis div marks *********************/
if (graph->yaxis_layout.scaling == XgLogarithm)
  {
    min = graph->yaxis_layout.sclmin;
    max = graph->yaxis_layout.sclmax;
    inc = 1.;
  }
else if (graph->yaxis_layout.scaling == XgProbability)
  {
    min = graph->yaxis_layout.prob_mindx;
    max = graph->yaxis_layout.prob_maxdx;
    inc = 1.;
  }
else if (graph->yaxis_layout.scaling == XgLinear)
  {
    /** get exp **/
    max = fabs(graph->yaxis_layout.sclmax);
    min = fabs(graph->yaxis_layout.sclmin);
    if (max > min)
      sprintf(buf, "%le", 0.9999 * max);
    else  /** the .9999 avoids a problem if max is an exact power of 10 **/
      sprintf(buf, "%le", 0.9999 * min);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    sscanf(chrptr, "%d", &exp);

    de = 0;
    if (graph->yaxis_layout.label_type == XgEng_label)
      {
	while (exp/3. != floor(exp/3.))
	  {
	    --exp;
	    ++de;
	  }
      }

    min = graph->yaxis_layout.sclmin;
    max = graph->yaxis_layout.sclmax;
    inc = pow(10., (double) ((exp + de)
			     - (graph->yaxis_layout.nsig_fig - XgSig0-1)));
    inc = (double) (graph->yaxis_layout.increment - XgLast0) * inc;
  }

x = graph->xaxis_layout.sclmin;
x1 = graph->xaxis_layout.sclmax;

for (y=min; y<max; y+=inc)
  {
    y1 = y;
    if (graph->yaxis_layout.scaling == XgProbability)
      y1 = Xg_inverf(XgProb_array[(int) y]);

    if ( ( (graph->yaxis_layout.div_marks == XgDivMajorTic)
	  || (graph->yaxis_layout.div_marks == XgDivMinorTic) )
	&& (y>min) && (y<max))
      {
	/** tick marks down left **/
	Xg_Mapwtod(x, y1, &nx, &ny, graph, XgLeft);
	XDrawLine(graph->display, XtWindow(graph->draw_area),
		  graph->theGC, nx, ny, nx + 10, ny);
	/** tick marks down right **/
	if (graph->use_right == FALSE)
	  {
	    Xg_Mapwtod(x1, y1, &nx, &ny, graph, XgLeft);
	    XDrawLine(graph->display, XtWindow(graph->draw_area),
		      graph->theGC, nx, ny, nx - 10, ny);
	  }
      }  /** end of Major Div Tics **/

    if ( ( (graph->yaxis_layout.div_marks == XgDivMajorLin)
	  || (graph->yaxis_layout.div_marks == XgDivMinorLin)
	  || (graph->yaxis_layout.div_marks == XgDivLin_Tic) )
	&& (y>min) && (y<max))
      {
/*	gcvalues.line_style = LineOnOffDash;
	XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);*/
	Xg_Mapwtod(x, y1, &nx, &ny, graph, XgLeft);
	Xg_Mapwtod(x1, y1, &nx1, &ny1, graph, XgLeft);
	XDrawLine(graph->display, XtWindow(graph->draw_area),
		  graph->theGC, nx, ny, nx1, ny1);
/*	gcvalues.line_style = LineSolid;
	XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);*/
      }  /** End of Major Div Lines **/

    if ((graph->yaxis_layout.div_marks == XgDivMinorTic)
	  || (graph->yaxis_layout.div_marks == XgDivLin_Tic))
      {
	if (graph->yaxis_layout.scaling == XgLogarithm)
	  {
	    dx = pow(10., y);
	    for (i=2; i<10; ++i)
	      {
		z = log10(i*dx);
		Xg_Mapwtod(x, z, &nx, &ny, graph, XgLeft);
		XDrawLine(graph->display, XtWindow(graph->draw_area),
			  graph->theGC, nx, ny, nx + 5, ny);
		if (graph->use_right == FALSE)
		  {
		    Xg_Mapwtod(x1, z, &nx, &ny, graph, XgLeft);
		    XDrawLine(graph->display, XtWindow(graph->draw_area),
			      graph->theGC, nx, ny, nx - 5, ny);
		  }
	      }
	  }  /** end of XgLogarithm **/
	else if (graph->yaxis_layout.scaling == XgLinear)
	  {
	    if (graph->yaxis_layout.increment == XgLast2)
	      j = 2;
	    else if (graph->yaxis_layout.increment == XgLast5)
	      j = 5;
	    else if (graph->yaxis_layout.increment == XgLast1)
	      j = 10;
	    dx = inc / (double) j;
	    for (i=1; i<j; ++i)
	      {
		z = y + dx*i;
		if ((z > min) && (z < max))
		  {
		    Xg_Mapwtod(x, z, &nx, &ny, graph, XgLeft);
		    XDrawLine(graph->display, XtWindow(graph->draw_area),
			      graph->theGC, nx, ny, nx + 5, ny);
		    if (graph->use_right == FALSE)
		      {
			Xg_Mapwtod(x1, z, &nx, &ny, graph, XgLeft);
			XDrawLine(graph->display, XtWindow(graph->draw_area),
				  graph->theGC, nx, ny, nx - 5, ny);
		      }
		  }
	      }
	  }  /** end of Linear **/
      }  /** end of Minor Div Tics **/

    if (graph->yaxis_layout.div_marks == XgDivMinorLin)
      {
	gcvalues.line_style = LineOnOffDash;
	XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);
	if (graph->yaxis_layout.scaling == XgLogarithm)
	  {
	    dx = pow(10., y);
	    for (i=2; i<10; ++i)
	      {
		z = log10(i*dx);
		Xg_Mapwtod(x, z, &nx, &ny, graph, XgLeft);
		Xg_Mapwtod(x1, z, &nx1, &ny1, graph, XgLeft);
		XDrawLine(graph->display, XtWindow(graph->draw_area),
			  graph->theGC, nx, ny, nx1, ny1);
	      }
	  }
	else if (graph->yaxis_layout.scaling == XgLinear)
	  {
	    if (graph->yaxis_layout.increment == XgLast2)
	      j = 2;
	    else if (graph->yaxis_layout.increment == XgLast5)
	      j = 5;
	    else if (graph->yaxis_layout.increment == XgLast1)
	      j = 10;
	    dx = inc / (double) j;
	    for (i=1; i<j; ++i)
	      {
		z = y + dx*i;
		if ((z > min) && (z < max))
		  {
		    Xg_Mapwtod(x, z, &nx, &ny, graph, XgLeft);
		    Xg_Mapwtod(x1, z, &nx1, &ny1, graph, XgLeft);
		    XDrawLine(graph->display, XtWindow(graph->draw_area),
			      graph->theGC, nx, ny, nx1, ny1);
		  }
	      }
	  }  /** end of Linear **/
	gcvalues.line_style = LineSolid;
	XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);
      }  /** end of Minor Div Lines **/
  }  /** end of y loop **/

/******************* y (right) axis div marks *********************/
if (graph->use_right == TRUE)
  {
    if (graph->y_rt_axis_layout.scaling == XgLogarithm)
      {
	min = graph->y_rt_axis_layout.sclmin;
	max = graph->y_rt_axis_layout.sclmax;
	inc = 1.;
      }
    else if (graph->y_rt_axis_layout.scaling == XgProbability)
      {
	min = graph->y_rt_axis_layout.prob_mindx;
	max = graph->y_rt_axis_layout.prob_maxdx;
	inc = 1.;
      }
    else if (graph->y_rt_axis_layout.scaling == XgLinear)
      {
	/** get exp **/
	max = fabs(graph->y_rt_axis_layout.sclmax);
	min = fabs(graph->y_rt_axis_layout.sclmin);
	if (max > min)
	  sprintf(buf, "%le", 0.9999 * max);
	else  /** the .9999 avoids a problem if max is an exact power of 10 **/
	  sprintf(buf, "%le", 0.9999 * min);
	chrptr = strchr(buf, 'e');
	++chrptr;
	sscanf(chrptr, "%d", &exp);

	de = 0;
	if (graph->y_rt_axis_layout.label_type == XgEng_label)
	  {
	    while (exp/3. != floor(exp/3.))
	      {
		--exp;
		++de;
	      }
	  }

	min = graph->y_rt_axis_layout.sclmin;
	max = graph->y_rt_axis_layout.sclmax;
	inc = pow(10., (double) ((exp + de)
			     - (graph->y_rt_axis_layout.nsig_fig - XgSig0-1)));
	inc = (double) (graph->y_rt_axis_layout.increment - XgLast0) * inc;
      }

    x = graph->xaxis_layout.sclmin;
    x1 = graph->xaxis_layout.sclmax;

    for (y=min; y<max; y+=inc)
      {
	y1 = y;
	if (graph->y_rt_axis_layout.scaling == XgProbability)
	  y1 = Xg_inverf(XgProb_array[(int) y]);

	if ( ( (graph->y_rt_axis_layout.div_marks == XgDivMajorTic)
	      || (graph->y_rt_axis_layout.div_marks == XgDivMinorTic) )
	    && (y>min) && (y<max))
	  {
	    /** tick marks down right **/
	    Xg_Mapwtod(x1, y1, &nx, &ny, graph, XgRight);
	    XDrawLine(graph->display, XtWindow(graph->draw_area),
		      graph->theGC, nx, ny, nx - 10, ny);
	  }  /** end of Major Div Tics **/

	if ( ( (graph->y_rt_axis_layout.div_marks == XgDivMajorLin)
	      || (graph->y_rt_axis_layout.div_marks == XgDivMinorLin)
	      || (graph->y_rt_axis_layout.div_marks == XgDivLin_Tic) )
	    && (y>min) && (y<max))
	  {
/*	    gcvalues.line_style = LineOnOffDash;
	    XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);*/
	    Xg_Mapwtod(x, y1, &nx, &ny, graph, XgRight);
	    Xg_Mapwtod(x1, y1, &nx1, &ny1, graph, XgRight);
	    XDrawLine(graph->display, XtWindow(graph->draw_area),
		      graph->theGC, nx, ny, nx1, ny1);
/*	    gcvalues.line_style = LineSolid;
	    XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);*/
	  }  /** end of Major Div Lines **/

	if ((graph->y_rt_axis_layout.div_marks == XgDivMinorTic)
	    || (graph->y_rt_axis_layout.div_marks == XgDivLin_Tic))
	  {
	    if (graph->y_rt_axis_layout.scaling == XgLogarithm)
	      {
		dx = pow(10., y);
		for (i=2; i<10; ++i)
		  {
		    z = log10(i*dx);
		    Xg_Mapwtod(x1, z, &nx, &ny, graph, XgRight);
		    XDrawLine(graph->display, XtWindow(graph->draw_area),
			      graph->theGC, nx, ny, nx - 5, ny);
		  }
	      }  /** end of XgLogarithm **/
	    else if (graph->y_rt_axis_layout.scaling == XgLinear)
	      {
		if (graph->y_rt_axis_layout.increment == XgLast2)
		  j = 2;
		else if (graph->y_rt_axis_layout.increment == XgLast5)
		  j = 5;
		else if (graph->y_rt_axis_layout.increment == XgLast1)
		  j = 10;
		dx = inc / (double) j;
		for (i=1; i<j; ++i)
		  {
		    z = y + dx*i;
		    if ((z > min) && (z < max))
		      {
			Xg_Mapwtod(x1, z, &nx, &ny, graph, XgRight);
			XDrawLine(graph->display, XtWindow(graph->draw_area),
				  graph->theGC, nx, ny, nx - 5, ny);
		      }
		  }
	      }  /** end of Linear **/
	  }  /** end of Minor Div Tics **/

	if (graph->y_rt_axis_layout.div_marks == XgDivMinorLin)
	  {
	    gcvalues.line_style = LineOnOffDash;
	    XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);
	    if (graph->y_rt_axis_layout.scaling == XgLogarithm)
	      {
		dx = pow(10., y);
		for (i=2; i<10; ++i)
		  {
		    z = log10(i*dx);
		    Xg_Mapwtod(x, z, &nx, &ny, graph, XgRight);
		    Xg_Mapwtod(x1, z, &nx1, &ny1, graph, XgRight);
		    XDrawLine(graph->display, XtWindow(graph->draw_area),
			      graph->theGC, nx, ny, nx1, ny1);
		  }
	      }
	    else if (graph->y_rt_axis_layout.scaling == XgLinear)
	      {
		if (graph->y_rt_axis_layout.increment == XgLast2)
		  j = 2;
		else if (graph->y_rt_axis_layout.increment == XgLast5)
		  j = 5;
		else if (graph->y_rt_axis_layout.increment == XgLast1)
		  j = 10;
		dx = inc / (double) j;
		for (i=1; i<j; ++i)
		  {
		    z = y + dx*i;
		    if ((z > min) && (z < max))
		      {
			Xg_Mapwtod(x, z, &nx, &ny, graph, XgRight);
			Xg_Mapwtod(x1, z, &nx1, &ny1, graph, XgRight);
			XDrawLine(graph->display, XtWindow(graph->draw_area),
				  graph->theGC, nx, ny, nx1, ny1);
		      }
		  }
	      }  /** end of Linear **/
	    gcvalues.line_style = LineSolid;
	    XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);
	  }  /** end of Minor Div Lines **/
      }  /** end of y loop **/
  }

}
