/**********************************************************************
*
*                            XgDrawLabels.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*  Draw Graph Labels
*
***********************************************************************
*/

#include "stdio.h"
#include "math.h"
#include "string.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

/**********************************************************************/
void Xg_drawLabels(XgGraphInfo * graph)
{
  int
    i,
    j,
    nxn,
    nyn,
    width,
    height;
  unsigned long
    value;
  DEV_COORD
    nx,
    ny;
  XWindowAttributes
    attrib;
  Pixmap
    thePixmap;
  XImage
    *theImage;

/*** x coord is center of string ***/

/******* x-axis label ********/
XSetForeground(graph->display, graph->theGC, graph->x_axis_desc.color);
nx = graph->x_axis_desc.coord.x * graph->nxdraw / 100.;
ny = graph->x_axis_desc.coord.y * graph->nydraw / 100.;
width = XTextWidth(graph->x_axis_desc.font_info.font_struct,
		 graph->x_axis_desc.text, strlen(graph->x_axis_desc.text));
height = graph->x_axis_desc.font_info.font_struct->ascent
  + graph->x_axis_desc.font_info.font_struct->descent;
nx -= width / 2;
/*ny -= height*/
graph->x_axis_desc.dimen.x = width;
graph->x_axis_desc.dimen.y = height;
XSetFont(graph->display, graph->theGC,
	 graph->x_axis_desc.font_info.font_struct->fid);
XDrawString(graph->display, XtWindow(graph->draw_area), graph->theGC, nx, ny,
	    graph->x_axis_desc.text, strlen(graph->x_axis_desc.text));

/******* y-axis label *********/
if (strlen(graph->y_axis_desc.text) > 0)
  {
    width = XTextWidth(graph->y_axis_desc.font_info.font_struct,
		       graph->y_axis_desc.text,
		       strlen(graph->y_axis_desc.text));
    height = graph->y_axis_desc.font_info.font_struct->ascent
      + graph->y_axis_desc.font_info.font_struct->descent;
    graph->y_axis_desc.dimen.x = height;
    graph->y_axis_desc.dimen.y = width;
    XGetWindowAttributes(graph->display, XtWindow(graph->draw_area), &attrib);
    thePixmap = XCreatePixmap(graph->display, XtWindow(graph->draw_area),
			  width, height, attrib.depth);
    XSetForeground(graph->display, graph->theGC, graph->background_color);
    XFillRectangle(graph->display, thePixmap, graph->theGC, 0,0, width,height);
    XSetForeground(graph->display, graph->theGC, graph->y_axis_desc.color);
    XSetFont(graph->display, graph->theGC,
	     graph->y_axis_desc.font_info.font_struct->fid);
    XDrawString(graph->display, thePixmap, graph->theGC, 0,
		height - graph->y_axis_desc.font_info.font_struct->descent,
		graph->y_axis_desc.text, strlen(graph->y_axis_desc.text));
    theImage = XGetImage(graph->display, thePixmap, 0, 0, width, height, 15,
			 XYPixmap);
    nx = graph->y_axis_desc.coord.x * graph->nxdraw / 100.;
    ny = graph->y_axis_desc.coord.y * graph->nydraw / 100.;
    ny += XTextWidth(graph->y_axis_desc.font_info.font_struct,
		     graph->y_axis_desc.text,
		     strlen(graph->y_axis_desc.text)) / 2;
    nx += graph->y_axis_desc.font_info.font_struct->ascent
      + graph->y_axis_desc.font_info.font_struct->descent;
    nyn = ny;
    for (i=0; i<width; ++i)
      {
	nxn = nx;
	for (j=height-1; j>=0; --j)
	  {
	    value = XGetPixel(theImage, i, j);
	    XSetForeground(graph->display, graph->theGC, value);
	    XDrawPoint(graph->display, XtWindow(graph->draw_area),
		       graph->theGC, nxn, nyn);
	    --nxn;
	  }
	--nyn;
      }
    XFreePixmap(graph->display, thePixmap);
  }

/******* y-axis (right) label *********/
if (graph->use_right == TRUE)
  {
    if (strlen(graph->y_rt_axis_desc.text) > 0)
      {
	width = XTextWidth(graph->y_rt_axis_desc.font_info.font_struct,
			   graph->y_rt_axis_desc.text,
			   strlen(graph->y_rt_axis_desc.text));
	height = graph->y_rt_axis_desc.font_info.font_struct->ascent
	  + graph->y_rt_axis_desc.font_info.font_struct->descent;
	graph->y_rt_axis_desc.dimen.x = height;
	graph->y_rt_axis_desc.dimen.y = width;
	XGetWindowAttributes(graph->display, XtWindow(graph->draw_area),
			     &attrib);
	thePixmap = XCreatePixmap(graph->display, XtWindow(graph->draw_area),
				  width, height, attrib.depth);
	XSetForeground(graph->display, graph->theGC, graph->background_color);
	XFillRectangle(graph->display, thePixmap, graph->theGC, 0, 0,
		       width, height);
	XSetForeground(graph->display, graph->theGC,
		       graph->y_rt_axis_desc.color);
	XSetFont(graph->display, graph->theGC,
		 graph->y_rt_axis_desc.font_info.font_struct->fid);
	XDrawString(graph->display, thePixmap, graph->theGC, 0,
		   height-graph->y_rt_axis_desc.font_info.font_struct->descent,
		    graph->y_rt_axis_desc.text,
		    strlen(graph->y_rt_axis_desc.text));
	theImage = XGetImage(graph->display, thePixmap, 0, 0, width, height,15,
			     XYPixmap);
	nx = graph->y_rt_axis_desc.coord.x * graph->nxdraw / 100.;
	ny = graph->y_rt_axis_desc.coord.y * graph->nydraw / 100.;
	ny += XTextWidth(graph->y_rt_axis_desc.font_info.font_struct,
			 graph->y_rt_axis_desc.text,
			 strlen(graph->y_rt_axis_desc.text)) / 2;
	nx += graph->y_rt_axis_desc.font_info.font_struct->ascent
	  + graph->y_rt_axis_desc.font_info.font_struct->descent;
	nyn = ny;
	for (i=width-1; i>=0; --i)
	  {
	    nxn = nx;
	    for (j=0; j<height; ++j)
	      {
		value = XGetPixel(theImage, i, j);
		XSetForeground(graph->display, graph->theGC, value);
		XDrawPoint(graph->display, XtWindow(graph->draw_area),
			   graph->theGC, nxn, nyn);
		--nxn;
	      }
	    --nyn;
	  }
	XFreePixmap(graph->display, thePixmap);
      }
  }

/********* title label **********/
XSetForeground(graph->display, graph->theGC, graph->title_desc.color);
nx = graph->title_desc.coord.x * graph->nxdraw / 100.;
ny = graph->title_desc.coord.y * graph->nydraw / 100.;
width = XTextWidth(graph->title_desc.font_info.font_struct,
		 graph->title_desc.text, strlen(graph->title_desc.text));
height = graph->title_desc.font_info.font_struct->ascent
  + graph->title_desc.font_info.font_struct->descent;
nx -= width / 2;
ny += height;
graph->title_desc.dimen.x = width;
graph->title_desc.dimen.y = height;
XSetFont(graph->display, graph->theGC,
	 graph->title_desc.font_info.font_struct->fid);
XDrawString(graph->display, XtWindow(graph->draw_area), graph->theGC,
	    nx, ny, graph->title_desc.text, strlen(graph->title_desc.text));

/********** subtitle label *********/
XSetForeground(graph->display, graph->theGC, graph->subtitle_desc.color);
nx = graph->subtitle_desc.coord.x * graph->nxdraw / 100.;
ny = graph->subtitle_desc.coord.y * graph->nydraw / 100.;
width = XTextWidth(graph->subtitle_desc.font_info.font_struct,
		 graph->subtitle_desc.text,
		 strlen(graph->subtitle_desc.text));
height = graph->subtitle_desc.font_info.font_struct->ascent
  + graph->subtitle_desc.font_info.font_struct->descent;
nx -= width / 2;
ny += height;
graph->subtitle_desc.dimen.x = width;
graph->subtitle_desc.dimen.y = height;
XSetFont(graph->display, graph->theGC,
	 graph->subtitle_desc.font_info.font_struct->fid);
XDrawString(graph->display, XtWindow(graph->draw_area), graph->theGC,
	    nx, ny, graph->subtitle_desc.text,
	    strlen(graph->subtitle_desc.text));

/******* dataset labels *********/
for (i=0; i<graph->num_sets; ++i)
  {
    XSetForeground(graph->display, graph->theGC,
		   graph->dataset[i].description.color);
    nx = graph->dataset[i].description.coord.x * graph->nxdraw / 100.;
    ny = graph->dataset[i].description.coord.y * graph->nydraw / 100.;
    width = XTextWidth(graph->dataset[i].description.font_info.font_struct,
		       graph->dataset[i].description.text,
		       strlen(graph->dataset[i].description.text));
    height = graph->dataset[i].description.font_info.font_struct->ascent
      + graph->dataset[i].description.font_info.font_struct->descent;
    ny += height;
    graph->dataset[i].description.dimen.x = width;
    graph->dataset[i].description.dimen.y = height;
    XSetFont(graph->display, graph->theGC,
	     graph->dataset[i].description.font_info.font_struct->fid);
    XDrawString(graph->display, XtWindow(graph->draw_area), graph->theGC,
		nx, ny, graph->dataset[i].description.text,
		strlen(graph->dataset[i].description.text));
  }  /** end of dataset label loop **/

}
