/**********************************************************************
*
*                            XgDrawLegend.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*  Draw Graph Legend
*
***********************************************************************
*/

#include "stdio.h"
#include "math.h"
#include "string.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"
#define Xg_LegendOffset  20

/**********************************************************************/
void Xg_drawLegend(XgGraphInfo * graph)
{
  int
    i,
    j,
    width,
    height,
    size,
    dimx,
    dimy;
  char
    buf[80];
  DEV_COORD
    nx,
    ny,
    nx0,
    ny0,
    xd,
    yd,
    dy;
  XPoint
    polypts[3];

if (graph->legend.use_legend != XgLegendOn)
  return;

/***** compute legend dimensions *****/
dimx = XTextWidth(graph->legend.description.font_info.font_struct,
		  graph->legend.description.text,
		  strlen(graph->legend.description.text));
height = graph->legend.description.font_info.font_struct->ascent
  + graph->legend.description.font_info.font_struct->descent;

j = 0;
for (i=0; i<graph->num_sets; ++i)
  {
    if (graph->dataset[i].in_legend == XgInLegend)
      {
	++j;
	strcpy(buf, graph->dataset[i].legend_text);
	strcat(buf, " ");
	if (graph->use_right == TRUE)
	  strcat(buf, " (L) ");
	width = XTextWidth(graph->legend.description.font_info.font_struct,
			   buf, strlen(buf));
	width += Xg_LegendOffset;     /** leave room for symbol **/
	if (width > dimx)
	  dimx = width;
      }
  }
if (j == 0)  /** no datasets in legend **/
  return;
dimy = (j+1.5) * height;
graph->legend.description.dimen.x = dimx;
graph->legend.description.dimen.y = dimy;
    
/******** draw box **********/
XSetForeground(graph->display, graph->theGC, graph->legend.description.color);
nx0 = graph->legend.description.coord.x * graph->nxdraw / 100.;
ny0 = graph->legend.description.coord.y * graph->nydraw / 100.;
nx = dimx;
ny = dimy;
XDrawRectangle(graph->display, XtWindow(graph->draw_area),
		   graph->theGC, nx0, ny0, dimx, dimy);

/******** set font, draw legend title **********/
XSetFont(graph->display, graph->theGC,
	 graph->legend.description.font_info.font_struct->fid);
width = XTextWidth(graph->legend.description.font_info.font_struct,
		  graph->legend.description.text,
		  strlen(graph->legend.description.text));
height = graph->legend.description.font_info.font_struct->ascent
  + graph->legend.description.font_info.font_struct->descent;
nx = nx0 + (dimx - width) / 2;
ny = ny0 + height;

XDrawString(graph->display, XtWindow(graph->draw_area), graph->theGC,
	    nx, ny, graph->legend.description.text,
	    strlen(graph->legend.description.text));

/******* draw dataset entries **********/
j = 1;
for (i=0; i<graph->num_sets; ++i)
  {
    if (graph->dataset[i].in_legend == XgInLegend)
      {
	++j;
	nx = nx0 + Xg_LegendOffset;
	ny = ny0 + j*height;

	/**** draw symbol ****/
	size = 5;
	dy = height / 4;
	xd = nx0 + Xg_LegendOffset / 2;
	yd = ny - dy;
	switch (graph->dataset[i].point_type)
	  {
	  case XgPoint:
	    XDrawPoint(graph->display, XtWindow(graph->draw_area),
		       graph->theGC, xd, yd);
	    break;
	  case XgCircle:
	    if (graph->dataset[i].filled_open == XgFilled)
	      XFillArc(graph->display, XtWindow(graph->draw_area),
		       graph->theGC, xd-size/2, yd-size/2, size, size,
		       0, 23040);
	    else if (graph->dataset[i].filled_open == XgOpen)
	      XDrawArc(graph->display, XtWindow(graph->draw_area),
		       graph->theGC, xd-size/2, yd-size/2, size, size,
		       0, 23040);
	    break;
	  case XgSquare:
	    if (graph->dataset[i].filled_open == XgFilled)
	      XFillRectangle(graph->display, XtWindow(graph->draw_area),
			     graph->theGC, xd-size/2, yd-size/2, size, size);
	    else if (graph->dataset[i].filled_open == XgOpen)
	      XDrawRectangle(graph->display, XtWindow(graph->draw_area),
			     graph->theGC, xd-size/2, yd-size/2, size, size);
	    break;
	  case XgTriangle:
	    if (graph->dataset[i].filled_open == XgFilled)
	      {
		polypts[0].x = xd - size/2;
		polypts[0].y = yd + size/2;
		polypts[1].x = xd + size/2;
		polypts[1].y = yd + size/2;
		polypts[2].x = xd;
		polypts[2].y = yd - size/2;
		XFillPolygon(graph->display, XtWindow(graph->draw_area),
			     graph->theGC, polypts, 3, Convex,
			     CoordModeOrigin);
	      }
	    else if (graph->dataset[i].filled_open == XgOpen)
		    {
		      XDrawLine(graph->display, XtWindow(graph->draw_area),
				graph->theGC, xd-size/2, yd+size/2,
				xd+size/2,yd+size/2);
		      XDrawLine(graph->display, XtWindow(graph->draw_area),
				graph->theGC, xd+size/2, yd+size/2,
				xd, yd-size/2);
		      XDrawLine(graph->display, XtWindow(graph->draw_area),
				graph->theGC, xd, yd-size/2,
				xd-size/2, yd+size/2);
		    }
	    break;
	  case XgPlus:
	    XDrawLine(graph->display, XtWindow(graph->draw_area),
		      graph->theGC, xd-size/2, yd, xd+size/2, yd);
	    XDrawLine(graph->display, XtWindow(graph->draw_area),
		      graph->theGC, xd, yd-size/2, xd, yd+size/2);
	    break;
	  case XgEx:
	    XDrawLine(graph->display, XtWindow(graph->draw_area),
		      graph->theGC, xd-size/2, yd+size/2, xd+size/2,yd-size/2);
	    XDrawLine(graph->display, XtWindow(graph->draw_area),
		      graph->theGC, xd-size/2, yd-size/2, xd+size/2,yd+size/2);
	    break;
	  case XgBar:
	    XFillRectangle(graph->display, XtWindow(graph->draw_area),
			   graph->theGC, xd-2, yd-2, 4, 6);
	    break;
	  default:
	    break;
	  }
	if (graph->dataset[i].line_status == XgSolidLine)
	  {
/*	    gcvalues.line_style = LineOnOffDash;
	    XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);*/
	    XDrawLine(graph->display, XtWindow(graph->draw_area),
		      graph->theGC, xd-size, yd-size/4,
		      xd+size, yd+size/4);
/*	    gcvalues.line_style = LineSolid;
	    XChangeGC(graph->display, graph->theGC, GCLineStyle, &gcvalues);*/
	  }

	/**** draw text ****/
	strcpy(buf, graph->dataset[i].legend_text);
	XDrawString(graph->display, XtWindow(graph->draw_area), graph->theGC,
		    nx, ny, buf, strlen(buf));
	if (graph->use_right == TRUE)
	  {
	    if (graph->dataset[i].left_right == XgLeft)
	      strcpy(buf, "(L) ");
	    else
	      strcpy(buf, "(R) ");
	    width = XTextWidth(graph->legend.description.font_info.font_struct,
			       buf, strlen(buf));
	    nx = nx0 + dimx - width;
	    XDrawString(graph->display, XtWindow(graph->draw_area),
			graph->theGC, nx, ny, buf, strlen(buf));
	  }  /** end of left/right indicator **/
      }  /** end of if dataset in legend **/
  }  /** end of i-loop over datasets **/

}
