/**********************************************************************
*
*                            XgHelpErrMisc.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*  Help & Error System, Miscellaneous Functions, Exit routine.
*
***********************************************************************
*/

#include "stdio.h"
#include "string.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "Xm/BulletinB.h"
#include "Xm/SelectioB.h"
#include "Xm/MessageB.h"
#include "Xm/Label.h"
#include "Xm/Text.h"

#include "XgPlot.h"
#include "XgPlot_loc.h"

/***********************************************************************
*
*                           Help & Error System
*
************************************************************************/

/**********************/
void Xg_main_help(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    rslt,
    count,
    nargs;
  char
    buf[80];
  FILE
    *fp;
  Arg
    args[5];
  XmString
    okaymsg,
    title,
    help_message[Xg_Max_Help];  /** make sure this is big enough **/
  Widget
    help_box,
    button;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;

fp = NULL;
sprintf(buf, "%s%s", XgDefHelpDir, graph->helpFile);
fp = fopen(buf, "r");
if (fp == NULL)
  {
    sprintf(buf, "%s", graph->helpFile);
    fp = fopen(buf, "r");
    if (fp == NULL)
      {
	sprintf(buf, "Unable to find Help File: %s", graph->helpFile);
	help_message[0]
	  = XmStringCreateLtoR(buf, XgCharset);
	count = 1;
      }
  }

if (fp != NULL)
  {
    count = 0;
    rslt = fscanf(fp, "%[^\n]", buf);
    getc(fp);
    while (rslt != EOF)
      {
	help_message[count] = XmStringCreateLtoR(buf, XgCharset);
	++count;
	buf[0] = '\0';   /** for blank lines **/
	rslt = fscanf(fp, "%[^\n]", buf);
	getc(fp);
	if (count >= Xg_Max_Help)
	  {
	    Xg_error_sys(Xg_HELP_FILE_TOO_BIG, graph);
	    rslt = EOF;
	  }
      }
    fclose(fp);
  }

title = XmStringCreateLtoR("Xg: Help", XgCharset);
okaymsg = XmStringCreateLtoR("I get it", XgCharset);
nargs = 0;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNlistItemCount, count); ++nargs;
XtSetArg(args[nargs], XmNlistItems, help_message); ++nargs;
XtSetArg(args[nargs], XmNlistVisibleItemCount, 12); ++nargs;
/*XtSetArg(args[nargs], XmNtextColumns, 40); ++nargs;*/
XtSetArg(args[nargs], XmNokLabelString, okaymsg); ++nargs;
/*XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;*/
help_box = (Widget) XmCreateSelectionDialog(graph->app_shell, "Xg: Help",
					    args, nargs);

button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_CANCEL_BUTTON);
XtUnmanageChild(button);
button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_HELP_BUTTON);
XtUnmanageChild(button);
button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_APPLY_BUTTON);
XtUnmanageChild(button);
button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_LIST_LABEL);
XtUnmanageChild(button);
button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(button);
button = (Widget) XmSelectionBoxGetChild(help_box, XmDIALOG_TEXT);
XtUnmanageChild(button);

for (rslt=0; rslt<count; ++rslt)
  XtFree(help_message[rslt]);
XtFree(okaymsg);
XtFree(title);
XtManageChild(help_box);
}

/*******************/
void Xg_help_sys(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  char
    help_message[5000];
  int
    nargs;
  Arg
    args[10];
  XmString
    okaymsg,
    title;
  Widget
    help_box,
    text_wid,
    button;

strcpy(help_message,
"             XgPlot Help\n\n");
/******* 45 char bar (window width) ********/

switch((int) client_data)
  {

  case XgHELP_TITLES:
    strcat(help_message,
"                Titles\n\n\
    Edit the label text and select the\n\
desired font for this label. Position of the\n\
label may be changed by entering the\n\
coordinates here (0-100) or by clicking and\n\
dragging the label with the mouse");
    break;

  case XgHELP_ADD_FILE:
    strcat(help_message,
"           Add a data file\n\n\
    Choose a file from the list to add\n\
to the list of data sets. Also indicate\n\
the type of file.\n\n\
    Normal has two lines of descriptive\n\
text, a line containing the number of data\n\
pairs and a single letter, followed by the\n\
ascii data pairs.\n\n\
    An ascii pairs file contains strictly\n\
(x,y) pairs of numbers (or triples,\n\
see below).\n\n\
    Single values files contain just y\n\
values, which will be plotted against\n\
their ordinal position.\n\n\
    If the data contains triplets\n\
(x,y,dy), available only for Normal files\n\
or ascii pairs, click the 'Has error data'\n\
button.");
    break;

  case XgHELP_SAVE_FILE:
    strcat(help_message,
"           Save data set to file\n\n\
    Select a file name for saving this\n\
dataset. Data will be saved in standard\n\
format with the leading two lines composed\n\
of the current dataset filename entry and\n\
the current dataset description.");
    break;

  case XgHELP_DATA_SELECT:
    strcat(help_message,
"        Choose the desired data set.");
    break;

  case XgHELP_FONTS:
    strcat(help_message,
"             Change Fonts\n\n\
    Select a new font (fonts have the\n\
extension .snf or .pcf, sometimes followed\n\
by .Z)");
    break;

  case XgHELP_DATA_OPTIONS:
    strcat(help_message,
"         Edit data set Options\n\n\
    You may edit the dataset annotation\n\
description, or change its font.\n\n\
    You may edit the legend description,\n\
and choose whether to include this\n\
dataset in the legend.\n\n\
    Point Style, Line Type, and Point Size\n\
select the appropriate values for this\n\
data set. The filled/open option refers\n\
to the circle and square point styles,\n\
thus effectively increasing the number\n\
of available point styles.\n\n\
    You may select to scale the\n\
data according to the left or right\n\
vertical axis. However, at least one\n\
data set must remain attached to the left\n\
vertical axis.\n\n\
    If the data contains error bar\n\
information, you may elect whether or not\n\
to display error bars.");
    break;

  case XgHELP_LOAD_SAVE:
    strcat(help_message,
"     Load or Save Graph Description\n\n\
    Choose a graph file to load or a name\n\
to save this graph to. As a convention,\n\
the extension *.grf is suggested.");
    break;

  case XgHELP_LAYOUT:
    strcat(help_message,
"              Axis Layout\n\n\
    Significant Digits selects the number\n\
of significant digits displayed by axis\n\
labels. Last Significant Digit indicates\n\
what the value of the last significant\n\
digit will be. Thus with data values\n\
0 to 9000, choosing Significant Digits\n\
as 2, and Last Significant Digit as 5\n\
will result in labels of 0, 500, 1000,\n\
1500, and so on.\n\n\
    The program automatically scales the\n\
data based on the minimum and maximum\n\
values in the data set. You may override\n\
this with the Minimum or Maximum box,\n\
either to exclude extremes of the data,\n\
or to force the scaling to match another\n\
graph.\n\n\
    Because the program does not know the\n\
magnitude of the data, it uses scientific\n\
notation by default. This may be toggled\n\
independently for each axis (note this\n\
only applies to linear scaling of data).\n\n\
    The default scaling is linear.\n\
Logarithmic or probability axis scaling is\n\
also available (probability is only\n\
available along the y axes).\n\n\
    You may choose several levels of\n\
subdivision markers within the graph region.\n\
Ticks marks or lines may be drawn\n\
perpendicular to the axis.");
    break;

  default:
    strcat(help_message, "          Internal Error: No such help");
    break;
  }
/******* 45 char bar (window width) ********/
strcat(help_message,
"\n\n              END OF HELP\n\n");

title = XmStringCreateLtoR("Xg: help", XgCharset);
okaymsg = XmStringCreateLtoR("I get it", XgCharset);
nargs = 0;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNokLabelString, okaymsg); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
help_box = XmCreatePromptDialog(w, "Xg: help", args, nargs);
button = XmSelectionBoxGetChild(help_box, XmDIALOG_CANCEL_BUTTON);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(help_box, XmDIALOG_HELP_BUTTON);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(help_box, XmDIALOG_APPLY_BUTTON);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(help_box, XmDIALOG_TEXT);
XtUnmanageChild(button);
button = XmSelectionBoxGetChild(help_box, XmDIALOG_SELECTION_LABEL);
XtUnmanageChild(button);

nargs = 0;
XtSetArg(args[nargs], XmNeditMode, XmMULTI_LINE_EDIT); ++nargs;
/*XtSetArg(args[nargs], XmNscrollBarDisplayPolicy, XmAS_NEEDED); ++nargs;
XtSetArg(args[nargs], XmNscrollingPolicy, XmAUTOMATIC); ++nargs;*/
XtSetArg(args[nargs], XmNcolumns, 45); ++nargs;
XtSetArg(args[nargs], XmNrows, 10); ++nargs;
XtSetArg(args[nargs], XmNscrollHorizontal, False); ++nargs;
XtSetArg(args[nargs], XmNscrollVertical, True); ++nargs;
XtSetArg(args[nargs], XmNeditable, FALSE); ++nargs;
text_wid = XmCreateScrolledText(help_box, "scrolled help", args, nargs);
XmTextSetString(text_wid, help_message);
XtManageChild(text_wid);

XtFree(okaymsg);
XtFree(title);
XtManageChild(help_box);
}

/******************* error system *************************/
void Xg_error_sys(error_type, graph)
     int
       error_type;
     XgGraphInfo
       *graph;
{
  int
    nargs;
  Arg
    args[10];
  XmString
    message,
    title,
    ok_message;
  Widget
    error_box,
    button;

switch (error_type)
  {
  case XgERR_NO_FILE:
    message = XmStringCreateLtoR("No Such File", XgCharset);
    break;

  case XgERR_DATA_NOT_FILE:
    message = XmStringCreateLtoR(
"   You cannot save this graph with a graph\n\
description file. One or more of the data\n\
sets is not from a data file.", XgCharset);
    break;

  case XgERR_DATA_CALLOC:
    message = XmStringCreateLtoR("Internal Error: Data calloc failure",
				 XgCharset);
    break;

  case XgERR_TOO_MANY_FILES:
    message = XmStringCreateLtoR("You are already using the maximum\n\
number of files. To add\nnew files you must delete\nsome existing ones",
XgCharset);
    break;

  case XgERR_NO_DELETE_LAST:
    message = XmStringCreateLtoR("You cannot delete the last data set\n\
Add another one, then delete this", XgCharset);
    break;

  case XgNOT_IMPLEMENTED:
    message = XmStringCreateLtoR("Function not implemented yet", XgCharset);
    break;

  case XgERR_NO_FONT:
    message = XmStringCreateLtoR("Unable to load this font", XgCharset);
    break;

  case XgERR_BAD_COORDS:
    message = XmStringCreateLtoR("       Invalid Coordinates\n\n\
Coordinates Must be between 0 and 100\nNo change made", XgCharset);
    break;

  case XgERR_BAD_MINMAX:
    message = XmStringCreateLtoR("       Min >= Max\n\n\
Minimum must be less than Maximum\nNo change made", XgCharset);
    break;

  case XgERR_LAST_Y_LEFT:
    message = XmStringCreateLtoR(
"    Last Left Y Axis Data\n\n\
You cannot delete the last dataset\n\
attached to the left hand y axis.\n\n\
To delete this data set, you must\n\
first assign another dataset to the\n\
left y axis.", XgCharset);
    break;

  case XgERR_BAD_LOG:
    message = XmStringCreateLtoR(
"A Value for the Logarithm is less than zero.\n\
Resetting to Linear Mode", XgCharset);
    break;

  case XgERR_BAD_PROBABILITY:
    message = XmStringCreateLtoR(
"A Value for the Inverse Probability is less than zero\n\
or greater than 1. Resetting to Linear Mode", XgCharset);
    break;

  case Xg_HELP_FILE_TOO_BIG:
    message = XmStringCreateLtoR("WARNING: Help file too big. Increase\n\
  Xg_Max_Help in XgPlot & recompile.\n\
  Rest of help has been ignored.", XgCharset);
    break;

  default:
    message = XmStringCreateLtoR("Internal Error: No such error!", XgCharset);
    break;
  }

nargs = 0;
XtSetArg(args[nargs], XmNmessageString, message); ++nargs;
ok_message = XmStringCreateLtoR("Continue", XgCharset);
title = XmStringCreateLtoR("Xg: Error System", XgCharset);
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNokLabelString, ok_message); ++nargs;
/*XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;*/
error_box = XmCreateErrorDialog(graph->app_shell, "Xg: Error System",
				args, nargs);
button = XmMessageBoxGetChild(error_box, XmDIALOG_HELP_BUTTON);
XtUnmanageChild(button);
button = XmMessageBoxGetChild(error_box, XmDIALOG_CANCEL_BUTTON);
XtUnmanageChild(button);
XtManageChild(error_box);
XtFree(message);
XtFree(title);
XtFree(ok_message);
}

/***********************************************************************
*
*                           Miscellaneous Functions
*
************************************************************************/

/***************************/
void Xg_cancel_CB(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
XtDestroyWidget(w);
if ((Widget) client_data != NULL)
  XtDestroyWidget((Widget) client_data);
}

/********************************/
int Xg_Getline
  (
   FILE *fp,
   char *buf
   )
{
  int
    rtrn,
    i;
  char
    c;
i = 0;
buf[0] = '\0';
c = getc(fp);
while ((c != (char) EOF) && (c != '\n'))
  {
    buf[i] = c;
    ++i;
    c = getc(fp);
  }
buf[i] = '\0';
rtrn = i;
if (c == (char) EOF)
  rtrn = EOF;
return(rtrn);
}

/******************************************************************
*
*                      Exit Graph
*
*******************************************************************/

/*************** Exit *************/
void Xg_create_exit_warning(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    nargs;
  Arg
    args[5];
  XmString
    title,
    okaymsg,
    warning_message;
  Widget
    warning_box,
    button;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
if (graph->kill_flag == XgNoKillOnExit)
  {
    warning_message = XmStringCreateLtoR("                        XgPlot\n\
You are about to exit the graph. Be sure anything you\n\
  want to save is saved.\n\
Press Exit to leave the graph and return to a\n\
  parent program.\n\
Press Cancel to return", XgCharset);
  }
else
  {
    warning_message = XmStringCreateLtoR("                        XgPlot\n\
You are about to exit the graph. Be sure anything you\n\
  want to save is saved.\n\
Press Exit to exit the program.\n\
Press Cancel to return", XgCharset);
  }

okaymsg = XmStringCreateLtoR("Exit", XgCharset);
title = XmStringCreateLtoR("Xg: Warning!", XgCharset);

nargs = 0;
XtSetArg(args[nargs], XmNdialogTitle, title); ++nargs;
XtSetArg(args[nargs], XmNmessageString, warning_message); ++nargs;
XtSetArg(args[nargs], XmNokLabelString, okaymsg); ++nargs;
XtSetArg(args[nargs], XmNautoUnmanage, False); ++nargs;
XtSetArg(args[nargs], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL); ++nargs;
warning_box = XmCreateWarningDialog(w, "Xg: Warning!", args, nargs);

button = XmMessageBoxGetChild(warning_box, XmDIALOG_HELP_BUTTON);
XtUnmanageChild(button);
XtAddCallback(warning_box, XmNcancelCallback, Xg_cancel_CB, NULL);
XtAddCallback(warning_box, XmNokCallback, Xg_exit_graph, client_data);

XtFree(warning_message);
XtFree(okaymsg);
XtFree(title);
XtManageChild(warning_box);
}

/**************************/
void Xg_exit_graph(w, client_data, call_data)
     Widget
       w;
     caddr_t
       client_data,
       call_data;
{
  int
    i;
  XgGraphInfo
    *graph;

graph = (XgGraphInfo *) client_data;
XtDestroyWidget(w);  /** the pop-up dialog box **/
XtDestroyWidget(graph->app_shell);
if (graph->cursor_status == XgCursOn)  /* cursor is on, turn it off */
  {
    Xg_delcurs(graph);
  }
for (i=0; i<graph->num_sets; ++i)
  {
    free(graph->dataset[i].darray);
    if (graph->dataset[i].err_array != NULL)
      free(graph->dataset[i].err_array);
  }
free(graph);

if (graph->kill_flag == XgKillOnExit)
  {
    exit(0);
  }

}
