/************************************************************************
*
*                             XgMapCoord.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*   Convert between data coordinates and normalized device coordinates
*
************************************************************************
*/
#include "stdio.h"
#include "math.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "XgPlot.h"
#include "XgPlot_loc.h"

/*************************************/
void Xg_Setworld
  (
   double xll,
   double yll,
   double xur,
   double yur,
   XgGraphInfo * graph,
   int left_right
   )
{
graph->sw_xll = xll;
graph->sw_xur = xur;
if (left_right == XgLeft)
  {
    graph->sw_yll = yll;
    graph->sw_yur = yur;
  }
else if (left_right == XgRight)
  {
    graph->sw_yll_rt = yll;
    graph->sw_yur_rt = yur;
  }
else
  fprintf(stderr, "ERROR: bad left_right flag in Xg_Setworld().\n");
}

/***************************/
void Xg_Mapwtod
  (
   double xw,
   double yw,
   DEV_COORD * xd,
   DEV_COORD * yd,
   XgGraphInfo * graph,
   int left_right
   )
{
*xd = (int) floor((xw - graph->sw_xll) / (graph->sw_xur - graph->sw_xll)
		  * graph->nxdraw + 0.5);
if (left_right == XgLeft)
  {
    *yd = graph->nydraw
      - (int) floor( (yw - graph->sw_yll) / (graph->sw_yur - graph->sw_yll)
		    * graph->nydraw + 0.5 );
  }
else if (left_right == XgRight)
  {
    *yd = graph->nydraw
      - (int) floor( (yw - graph->sw_yll_rt)
		    / (graph->sw_yur_rt - graph->sw_yll_rt)
		    * graph->nydraw + 0.5 );
  }
else
  fprintf(stderr, "ERROR: bad left_right flag in Xg_Mapwtod().\n");
}

/**********************/
void XgMapdtow
  (
   int xd,
   int yd,
   double * xw,
   double * yw,
   XgGraphInfo * graph,
   int left_right
   )
{
*xw = graph->sw_xll + (graph->sw_xur - graph->sw_xll) *
  (double) (xd) / (double) (graph->nxdraw);
if (left_right == XgLeft)
  {
    *yw = (double) (graph->nydraw - yd) / (double) (graph->nydraw)
      * (graph->sw_yur - graph->sw_yll) + graph->sw_yll;
  }
else if (left_right == XgRight)
  {
    *yw = (double) (graph->nydraw - yd) / (double) (graph->nydraw)
      * (graph->sw_yur_rt - graph->sw_yll_rt) + graph->sw_yll_rt;
  }
else
  fprintf(stderr, "ERROR: bad left_right flag in Xg_Mapdtow().\n");
}
