/**********************************************************************
*
*                            XgSclData.c
*
*  Copyright (c) 1992, 1993
*      John N. Sanders
*      Massachusetts Institute of Technology
*      Lincoln Laboratory
*      244 Wood Street
*      Lexington, MA 02173.
*  All Rights Reserved.
*  Version 3.0   5/93
*
*  Perform data scaling to find min/max along each axis, adjust to fit
*   rational labels. Check linear, log, or probability scaling.
*
***********************************************************************
*/

#include "stdio.h"
#include "math.h"
#include "string.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "XgPlot.h"
#include "XgPlot_loc.h"

/****************************************/
void Xg_scale_data(graph)
     XgGraphInfo
       *graph;
{
  int
    i,
    j,
    k;
  double
    inc,
    xmin,
    xmax,
    ymin,
    ymax,
    ylmin,
    ylmax,
    yrmin,
    yrmax;
  char
    buf[20],
    cnum[20],
    *chrptr;

/*************** set min, max values *************/
xmin = 1e+12;
xmax = -1e+12;
ylmin = 1e+12;
ylmax = -1e+12;
yrmin = 1e+12;
yrmax = -1e+12;

graph->use_right = FALSE;
k = 0;

for (j=0; j<graph->num_sets; ++j)
  {
    for (i=0; i<graph->dataset[j].num_points; ++i)
      {
	if (*(graph->dataset[j].darray + 2*i) > xmax)
	  xmax = *(graph->dataset[j].darray + 2*i);
	if (*(graph->dataset[j].darray + 2*i) < xmin)
	  xmin = *(graph->dataset[j].darray + 2*i);
	if (graph->dataset[j].left_right == XgLeft)
	  {
	    if (*(graph->dataset[j].darray + 2*i + 1) > ylmax)
	      ylmax = *(graph->dataset[j].darray + 2*i + 1);
	    if (*(graph->dataset[j].darray + 2*i + 1) < ylmin)
	      ylmin = *(graph->dataset[j].darray + 2*i + 1);
	  }
	else  /** XgRight **/
	  {
	    ++k;
	    if (*(graph->dataset[j].darray + 2*i + 1) > yrmax)
	      yrmax = *(graph->dataset[j].darray + 2*i + 1);
	    if (*(graph->dataset[j].darray + 2*i + 1) < yrmin)
	      yrmin = *(graph->dataset[j].darray + 2*i + 1);
	  }
      }
  }

if (!(graph->xaxis_layout.user_set_flag & XgUmin))
  graph->xaxis_layout.min = xmin;
if (!(graph->xaxis_layout.user_set_flag & XgUmax))
  graph->xaxis_layout.max = xmax;
if (!(graph->yaxis_layout.user_set_flag & XgUmin))
  graph->yaxis_layout.min = ylmin;
if (!(graph->yaxis_layout.user_set_flag & XgUmax))
  graph->yaxis_layout.max = ylmax;

if (k > 0)  /** only valid if there are datasets attached to right axis **/
  {
    graph->use_right = TRUE;
    if (!(graph->y_rt_axis_layout.user_set_flag & XgUmin))
      graph->y_rt_axis_layout.min = yrmin;
    if (!(graph->y_rt_axis_layout.user_set_flag & XgUmax))
      graph->y_rt_axis_layout.max = yrmax;
  }

/************** adjust to labeling parameters ************/
/** We do this adjustment regardless of XgUmax/min flag **/

/****************** x-axis **************/
if (graph->xaxis_layout.scaling == XgLogarithm)
  {
    if (graph->xaxis_layout.min <= 0.) /** check for validity **/
      {
	Xg_error_sys(XgERR_BAD_LOG, graph);
	graph->xaxis_layout.scaling = XgLinear;
      }
    else
      {
	graph->xaxis_layout.sclmin
	  = floor(log10(graph->xaxis_layout.min));
	graph->xaxis_layout.sclmax
	  = ceil(log10(graph->xaxis_layout.max));
      }
  }

if (graph->xaxis_layout.scaling == XgLinear)
  {
    inc = graph->xaxis_layout.increment - XgLast0;
    sprintf(buf, "%le", graph->xaxis_layout.min); /*get sci notation form*/
    sscanf(buf, "%[^e]", cnum);
    sscanf(cnum, "%le", &xmin);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%le", &ymin);
    sprintf(buf, "%le", graph->xaxis_layout.max);
    sscanf(buf, "%[^e]", cnum);
    sscanf(cnum, "%le", &xmax);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%le", &ymax);
    if (graph->xaxis_layout.nsig_fig == XgSig2)
      { /* scale to proper sig. digit */
	xmin = xmin * 10;
	xmax = xmax * 10;
      }
    if (graph->xaxis_layout.nsig_fig == XgSig3)
      {
	xmin = xmin * 100;
	xmax = xmax * 100;
      }
    if ((xmax != 0) && (xmin != 0))
      {
	if (ymax > ymin)          /* adjust both to same scale */
	  xmin = xmin / pow(10., ymax - ymin);
	else if (ymax < ymin)
	  xmax = xmax / pow(10., ymin - ymax);
      }
        /* move up or down to next level */
    xmin = floor(xmin / inc) * inc;
    xmax = ceil(xmax / inc) * inc;
    if ((xmax != 0) && (xmin != 0))
      {
	if (ymax > ymin)       /* restore to different scales */
	  xmin = xmin * pow(10., ymax - ymin);
	else if (ymax < ymin)
	  xmax = xmax * pow(10., ymin - ymax);
      }
    if (graph->xaxis_layout.nsig_fig == XgSig2) /* undo sig. digit shift */
      {
	xmin = xmin / 10;
	xmax = xmax / 10;
      }
    if (graph->xaxis_layout.nsig_fig == XgSig3)
      {
	xmin = xmin / 100;
	xmax = xmax / 100;
      }
         /* restore power of 10 */
    graph->xaxis_layout.min = xmin * pow(10., ymin);
    graph->xaxis_layout.max = xmax * pow(10., ymax);

    /************** include zero if nearby *******************/
    xmax = graph->xaxis_layout.max - graph->xaxis_layout.min;
    if (!(graph->xaxis_layout.user_set_flag & XgUmin))
      {
	if ((graph->xaxis_layout.min > 0) &&
	    (graph->xaxis_layout.min < xmax/2.))
	  graph->xaxis_layout.min = 0.;
      }
    if (!(graph->xaxis_layout.user_set_flag & XgUmax))
      {
	if ((graph->xaxis_layout.max < 0) &&
	    (-graph->xaxis_layout.max < xmax/2.))
	  graph->xaxis_layout.max = 0.;
      }
    graph->xaxis_layout.sclmin = graph->xaxis_layout.min;
    graph->xaxis_layout.sclmax = graph->xaxis_layout.max;
  }  /** end of x-axis Linear **/

/****************** y-axis (Left) *********************/
if (graph->yaxis_layout.scaling == XgLogarithm)
  {
    if (graph->yaxis_layout.min <= 0.) /** check for validity **/
      {
	Xg_error_sys(XgERR_BAD_LOG, graph);
	graph->yaxis_layout.scaling = XgLinear;
      }
    else
      {
	graph->yaxis_layout.sclmin
	  = floor(log10(graph->yaxis_layout.min));
	graph->yaxis_layout.sclmax
	  = ceil(log10(graph->yaxis_layout.max));
      }
  }

if (graph->yaxis_layout.scaling == XgProbability)
  {
    if ((graph->yaxis_layout.min < 0.)   /** check for validity **/
	|| (graph->yaxis_layout.max > 1.))
      {
	Xg_error_sys(XgERR_BAD_PROBABILITY, graph);
	graph->yaxis_layout.scaling = XgLinear;
      }
    else
      {
	graph->yaxis_layout.prob_mindx = 0;
	graph->yaxis_layout.prob_maxdx = XgNPROB_ENT;
	for (i=0; i<=XgNPROB_ENT; ++i)
	  {
	    if (graph->yaxis_layout.min >= XgProb_array[i])
	      graph->yaxis_layout.prob_mindx = i;
	    if (graph->yaxis_layout.max <= XgProb_array[XgNPROB_ENT - i])
	      graph->yaxis_layout.prob_maxdx = XgNPROB_ENT - i;
	  }
	graph->yaxis_layout.sclmin
	  = Xg_inverf(XgProb_array[graph->yaxis_layout.prob_mindx]);
	graph->yaxis_layout.sclmax
	  = Xg_inverf(XgProb_array[graph->yaxis_layout.prob_maxdx]);
      }
  }

if (graph->yaxis_layout.scaling == XgLinear)
  {
    inc = graph->yaxis_layout.increment - XgLast0;
       /* get sci notation form */
    sprintf(buf, "%le", graph->yaxis_layout.min);
    sscanf(buf, "%[^e]", cnum);
    sscanf(cnum, "%le", &ymin);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%le", &xmin);
    sprintf(buf, "%le", graph->yaxis_layout.max);
    sscanf(buf, "%[^e]", cnum);
    sscanf(cnum, "%le", &ymax);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%le", &xmax);
        /* scale to proper sig. digit*/
    if (graph->yaxis_layout.nsig_fig == XgSig2)
      {
	ymin = ymin * 10;
	ymax = ymax * 10;
      }
    if (graph->yaxis_layout.nsig_fig == XgSig3)
      {
	ymin = ymin * 100;
	ymax = ymax * 100;
      }
    if ((ymax != 0) && (ymin != 0))
      {
	if (xmax > xmin)          /* adjust both to same scale */
	  ymin = ymin / pow(10., xmax - xmin);
	else if (xmax < xmin)
	  ymax = ymax / pow(10., xmin - xmax);
      }
       /* move up or down to next level */
    ymin = floor(ymin / inc) * inc;
    ymax = ceil(ymax / inc) * inc;
    if ((ymax != 0) && (ymin != 0))
      {
	if (xmax > xmin)       /* restore to different scales */
	  ymin = ymin * pow(10., xmax - xmin);
	else if (xmax < xmin)
	  ymax = ymax * pow(10., xmin - xmax);
      }
    if (graph->yaxis_layout.nsig_fig == XgSig2) /* undo sig. digit shift */
      {
	ymin = ymin / 10;
	ymax = ymax / 10;
      }
    if (graph->yaxis_layout.nsig_fig == XgSig3)
      {
	ymin = ymin / 100;
	ymax = ymax / 100;
      }
         /* restore power of 10 */
    graph->yaxis_layout.min = ymin * pow(10., xmin);
    graph->yaxis_layout.max = ymax * pow(10., xmax);

    /************** include zero if nearby *******************/
    ymax = graph->yaxis_layout.max - graph->yaxis_layout.min;
    if (!(graph->yaxis_layout.user_set_flag & XgUmin))
      {
	if ((graph->yaxis_layout.min > 0) &&
	    (graph->yaxis_layout.min < ymax/2.))
	  graph->yaxis_layout.min = 0.;
      }
    if (!(graph->yaxis_layout.user_set_flag & XgUmax))
      {
	if ((graph->yaxis_layout.max < 0) &&
	    (-graph->yaxis_layout.max < ymax/2.))
	  graph->yaxis_layout.max = 0.;
      }
    graph->yaxis_layout.sclmin = graph->yaxis_layout.min;
    graph->yaxis_layout.sclmax = graph->yaxis_layout.max;
  } /** end of y-axis Linear **/

/****************** y-axis (Right) *********************/
if (graph->use_right == TRUE)
  {       /** only scale this axis if there are datasets attached to it **/

if (graph->y_rt_axis_layout.scaling == XgLogarithm)
  {
    if (graph->y_rt_axis_layout.min <= 0.) /** check for validity **/
      {
	Xg_error_sys(XgERR_BAD_LOG, graph);
	graph->y_rt_axis_layout.scaling = XgLinear;
      }
    else
      {
	graph->y_rt_axis_layout.sclmin
	  = floor(log10(graph->y_rt_axis_layout.min));
	graph->y_rt_axis_layout.sclmax
	  = ceil(log10(graph->y_rt_axis_layout.max));
      }
  }

if (graph->y_rt_axis_layout.scaling == XgProbability)
  {
    if ((graph->y_rt_axis_layout.min < 0.)   /** check for validity **/
	|| (graph->y_rt_axis_layout.max > 1.))
      {
	Xg_error_sys(XgERR_BAD_PROBABILITY, graph);
	graph->y_rt_axis_layout.scaling = XgLinear;
      }
    else
      {
	graph->y_rt_axis_layout.prob_mindx = 0;
	graph->y_rt_axis_layout.prob_maxdx = XgNPROB_ENT;
	for (i=0; i<=XgNPROB_ENT; ++i)
	  {
	    if (graph->y_rt_axis_layout.min >= XgProb_array[i])
	      graph->y_rt_axis_layout.prob_mindx = i;
	    if (graph->y_rt_axis_layout.max <= XgProb_array[XgNPROB_ENT - i])
	      graph->y_rt_axis_layout.prob_maxdx = XgNPROB_ENT - i;
	  }
	graph->y_rt_axis_layout.sclmin
	  = Xg_inverf(XgProb_array[graph->y_rt_axis_layout.prob_mindx]);
	graph->y_rt_axis_layout.sclmax
	  = Xg_inverf(XgProb_array[graph->y_rt_axis_layout.prob_maxdx]);
      }
  }

if (graph->y_rt_axis_layout.scaling == XgLinear)
  {
    inc = graph->y_rt_axis_layout.increment - XgLast0;
       /* get sci notation form */
    sprintf(buf, "%le", graph->y_rt_axis_layout.min);
    sscanf(buf, "%[^e]", cnum);
    sscanf(cnum, "%le", &ymin);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%le", &xmin);
    sprintf(buf, "%le", graph->y_rt_axis_layout.max);
    sscanf(buf, "%[^e]", cnum);
    sscanf(cnum, "%le", &ymax);
    chrptr = strchr(buf, 'e');
    ++chrptr;
    if (*chrptr == '+')  /** scanf on some systems choke on nums with + **/
      ++chrptr;
    sscanf(chrptr, "%le", &xmax);
        /* scale to proper sig. digit*/
    if (graph->y_rt_axis_layout.nsig_fig == XgSig2)
      {
	ymin = ymin * 10;
	ymax = ymax * 10;
      }
    if (graph->y_rt_axis_layout.nsig_fig == XgSig3)
      {
	ymin = ymin * 100;
	ymax = ymax * 100;
      }
    if ((ymax != 0) && (ymin != 0))
      {
	if (xmax > xmin)          /* adjust both to same scale */
	  ymin = ymin / pow(10., xmax - xmin);
	else if (xmax < xmin)
	  ymax = ymax / pow(10., xmin - xmax);
      }
       /* move up or down to next level */
    ymin = floor(ymin / inc) * inc;
    ymax = ceil(ymax / inc) * inc;
    if ((ymax != 0) && (ymin != 0))
      {
	if (xmax > xmin)       /* restore to different scales */
	  ymin = ymin * pow(10., xmax - xmin);
	else if (xmax < xmin)
	  ymax = ymax * pow(10., xmin - xmax);
      }
    if (graph->y_rt_axis_layout.nsig_fig == XgSig2) /* undo sig. digit shift */
      {
	ymin = ymin / 10;
	ymax = ymax / 10;
      }
    if (graph->y_rt_axis_layout.nsig_fig == XgSig3)
      {
	ymin = ymin / 100;
	ymax = ymax / 100;
      }
         /* restore power of 10 */
    graph->y_rt_axis_layout.min = ymin * pow(10., xmin);
    graph->y_rt_axis_layout.max = ymax * pow(10., xmax);

    /************** include zero if nearby *******************/
    ymax = graph->y_rt_axis_layout.max - graph->y_rt_axis_layout.min;
    if (!(graph->y_rt_axis_layout.user_set_flag & XgUmin))
      {
	if ((graph->y_rt_axis_layout.min > 0) &&
	    (graph->y_rt_axis_layout.min < ymax/2.))
	  graph->y_rt_axis_layout.min = 0.;
      }
    if (!(graph->y_rt_axis_layout.user_set_flag & XgUmax))
      {
	if ((graph->y_rt_axis_layout.max < 0) &&
	    (-graph->y_rt_axis_layout.max < ymax/2.))
	  graph->y_rt_axis_layout.max = 0.;
      }
    graph->y_rt_axis_layout.sclmin = graph->y_rt_axis_layout.min;
    graph->y_rt_axis_layout.sclmax = graph->y_rt_axis_layout.max;
  } /** end of y-axis Linear **/
}  /** end of if (graph->use_right == TRUE) test for right axis datasets **/

}

/**********************************************************************
*
*                              Xg_inverf()
*
* Originally written by S.C. Pohlig, adapted by J.N. Sanders
*
* This function returns an approximation to the inverse of the standard
* normal probability distribution.  The approximation error is less than
* 4.5e-4.  The approximation formula is from M. Abramowitz and I. A. Stegun,
* Handbook of Mathematical Functions, eqn. 26.2.23, Dover Publications, Inc.
*
* The C language error function returns erf(x) = (2/sqrt(pi)) * Integral(0,x)
*   of exp(-t*t)dt, which gives erf(infinity) = 1. In essence, this gives the
*   area under the curve between -x and +x, normalized to 1.
* However, this function (inverf), solves for the inverse of (1/sqrt(pi))
*   * Integral(-infinity, x) of exp(-t*t)dt.
* As a result, the symmetric inverse is:
*      x = inverf(erf(x) / 2. + .5)
*
* Given the integral of a unit variance gaussian, from -infinity to x,
*  normalized such that the integral to +infinity is 1, multiply this result
*  by sqrt(2) to obtain x.
*
**********************************************************************
*/

#define	C0	2.515517
#define	C1	0.802853
#define	C2	0.010328
#define	D1	1.432788
#define	D2	0.189269
#define	D3	0.001308
#define MAX_SIGMA      7

double Xg_inverf   /** 0 <= p <= 1 **/
  (
   double p
  )
{
	double	t1, t2, q, x;

	if (p >= 1.)		return(MAX_SIGMA);
	else if (p <= 0.)	return(-MAX_SIGMA);
	else if (p == 0.5)	return(0.0);

	if (p < 0.5)	q = p;
	else		q = 1.0 - p;

	t2 = -2.0 * log(q);
	t1 = sqrt(t2);

	x = t1 - (C0 + C1*t1 + C2*t2)/(1.0 + D1*t1 + D2*t2 + D3*t1*t2);
#ifndef ERF_X_DIV_ROOT2
	x = x / sqrt(2.);   /** jns **/
#endif

	if (p < 0.5)	return(-x);
	else		return(x);
}
