/*********************************************************************
*
*                          pltxsh.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.2   11/92
*
**********************************************************************
*/
#include "stdio.h"
#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"
#include "XgPlot.h"
#define FILE_MAX 1024

main(argc, argv)
     int
       argc;
     char
       *argv[];
{
  Widget
    graph;
  Display
    *display;
  XgGraphInfo
    *graphinfo;
  FILE
    *fp;
  char
    filename[80],
    buf[80];
  int
    i,
    n,
    d_type,
    l_type,
    f_type,
    err_type;
  double
    *data;

graphinfo = XgCreateStruct();

/************* get filename if on command line ******************/
fp = NULL;
for (i=1; i<argc; ++i) /* find file name if given */
  {
    if ((!strcmp(argv[i], "-p")) || (!strcmp(argv[i], "-s"))
	|| (!strcmp(argv[i], "-l")));
    else if (!strncmp(argv[i], "-", 1))
      ++i;
    else /** assume this is file name **/
      {
	strcpy(filename, argv[i]);
	fp = fopen(filename, "r");
	i = argc;
      }
  }

/*********** get command line options ************/
d_type = XgCircle;
l_type = XgNoLine;
f_type = Xg_Add_Normal;
err_type = XgNoErrorPts;

for (i=1; i<argc; ++i)  /* get command line options */
  {
    if (!strncmp(argv[i], "-l", 2)) /** turn on line **/
      l_type = XgSolidLine;
    if (!strncmp(argv[i], "-p", 2)) /** just ascii pairs of numbers **/
      f_type = Xg_Add_Pair;
    if (!strncmp(argv[i], "-s", 2)) /** just ascii single numbers **/
      f_type = Xg_Add_Single;
    if (!strncmp(argv[i], "-e", 2)) /** has error bar triplets **/
      err_type = XgHasErrorPts;
    if (!strncmp(argv[i], "-g", 2)) /** use a description file **/
      {
	f_type = 4;
	sscanf(argv[i+1], "%s", buf);
	fp = fopen(buf, "r");
	if (fp == NULL)
	  {
	    fprintf(stderr, "Can't open description file. Aborting!\n");
	    exit(1);
	  }
      }
    if (!strncmp(argv[i], "-t", 2)) /** Set data point type **/
      {
	sscanf(argv[i+1], "%s", buf);
	if (buf[0] == 's')
	  d_type = XgSquare;
	if (buf[0] == 't')
	  d_type = XgTriangle;
	if (buf[0] == 'b')
	  d_type = XgBar;
	if (buf[0] == 'p')
	  d_type = XgPlus;
	if (buf[0] == 'x')
	  d_type = XgEx;
      }
    if (!strncmp(argv[i], "-h", 2)) /** check for help **/
      {
	pltxhlp(argv[0]);
	exit(0);
      }
  }

graphinfo->num_sets = 1;
graphinfo->dataset[0].line_status = l_type;
graphinfo->dataset[0].point_type = d_type;

/********** open file, read data **********/
while (fp == NULL)
  {
    printf("Enter file name to plot: ");
    scanf("%[^\n]", filename);
    getchar();
    fp = fopen(filename, "r");
    if (fp == NULL)
      {
	fprintf(stderr, "No such file!\nTry Again Twinkie\n");
      }
  }

if (f_type == 4)
  {
    XgReadGrf(fp, graphinfo);
  }
else
  {
    graphinfo->dataset[0].file_type = f_type;
    graphinfo->dataset[0].error_pts = err_type;
    Xg_read_datafile(fp, graphinfo, 0);
    strcpy(graphinfo->dataset[0].filename, filename);
    strcpy(graphinfo->dataset[0].legend_text, filename);
  }

fclose(fp);

#ifdef TEXT
printf("Enter an x-axis description: ");
scanf("%[^\n]", graphinfo->x_axis_desc.text);
getchar();
printf("Enter a y-axis description: ");
scanf("%[^\n]", graphinfo->y_axis_desc.text);
getchar();
printf("Enter a graph title: ");
scanf("%[^\n]", graphinfo->title_desc.text);
getchar();
#endif

/********************************/
XtToolkitInitialize();
graphinfo->theAppContext = XtCreateApplicationContext();
display = XtOpenDisplay(graphinfo->theAppContext, NULL, argv[0], "XgPlot",
			NULL, 0, (Cardinal *) &argc, argv);
if (display == NULL)
  {
    fprintf(stderr, "Unable to open requested display. Aborting\n");
    exit(1);
  }
graphinfo->app_shell = XtAppCreateShell("Pltxsh", XgClass,
					applicationShellWidgetClass,
					display, NULL, 0);
XgCreateGraph(graphinfo);
XtAppMainLoop(graphinfo->theAppContext);
}

/********************/
pltxhlp(execname)
     char
       *execname;
{
fprintf(stderr, "%s command line help:\n", execname);
fprintf(stderr, " usage: %s [options]\n", execname);
fprintf(stderr, "      where options are one or more of the following:\n");
fprintf(stderr, "   any of the standard Xt command line options\n");
fprintf(stderr, "   [-l] turns on line connecting data points\n");
fprintf(stderr, "   [-t (type)] selects data point representation where\n");
fprintf(stderr, "      type may be: (s)quare, (t)riangle, (b)ar, (p)lus\n");
fprintf(stderr, "      or e(x)\n");
fprintf(stderr, "   [(a data file name)] indicating startup data to plot\n");
fprintf(stderr, "   [-g (description file name)] use a stored layout\n");
fprintf(stderr, "   [-p] The data is just ascii pairs of numbers (x y)\n");
fprintf(stderr, "   [-s] The data is just ascii y values to be plotted\n");
fprintf(stderr, "   [-e] The data is triples, (x,y,err) for error bars\n");
fprintf(stderr, "        This option is not valid with [-s]\n");
fprintf(stderr, "\n   The default data format contains a header described below:\n");
fprintf(stderr, "     Two lines of descriptive text, each terminated by a\n");
fprintf(stderr, "     newline character. A third line containing an ascii\n");
fprintf(stderr, "     number followed by a space and a single letter.\n");
fprintf(stderr, "     The number is the number of x,y data pairs following\n");
fprintf(stderr, "     and the letter is not used (it is for other\n");
fprintf(stderr, "     applications). Thus a file with three data pairs\n");
fprintf(stderr, "     might look like:\n");
fprintf(stderr, "         First line of comment\n");
fprintf(stderr, "         Second line of comment\n");
fprintf(stderr, "         3 T\n");
fprintf(stderr, "         1 5\n");
fprintf(stderr, "         2 10\n");
fprintf(stderr, "         3 15\n");
}
