/**********************************************************************
*
*                              slbdfunc.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 1.0   1/94
*
*  Program for performing I/O operations on standard SLBD data files.
*
***********************************************************************
*/
#include "stdio.h"
#include "string.h"
#include "etsheadr.h"

#ifdef sun
#define NON_ANSI
#endif

/* define IS_IEEE_MACHINE for 680X0 and all RISC machines (Sun, SGI, etc.),
   not for DEC or Intel */
#ifndef _I386
#ifndef _DEC
#ifndef __alpha
#define IS_IEEE_MACHINE
#endif
#endif
#endif

/***************** open data file *********************/
#ifdef NON_ANSI
void slbdopen(datafile)
    ETS_STRUCT
      *datafile;
#else
void slbdopen
  (
   ETS_STRUCT * datafile
  )
#endif
{
  int
    nframes;
  char
    *header_char,
    sbyte,
    inp_frmt[10],
    tbuf[300];
  struct
    {
      int start_frame;
      int end_frame;
      int ncols;
      int nrows;
      int frm_len;
    } slbd_hdr;

/*************** check basics for proper definitions, etc. ************/
#ifdef IS_IEEE_MACHINE
    datafile->this_machine = IEEE_MACHINE;
#else
    datafile->this_machine = NON_IEEE_MACHINE;
#endif
if (strlen(datafile->filename) == 0)  /* no predefined name */
  printf("file header is %d bytes long, according to etsheadr.h\n",
	 sizeof(datafile->file_header));
if (HEADER_SIZE != sizeof(datafile->file_header))
  {
    fprintf(stderr, "**** HEADER_SIZE = %d but sizeof(file_header) = %d\n",
      HEADER_SIZE, sizeof(datafile->file_header));
  }
if (sizeof(int2) != 2)
  {
    fprintf(stderr, "**** int2 size defined wrong in etsheadr.h! Aborting\n");
    exit(1);
  }
if (sizeof(int4) != 4)
  {
    fprintf(stderr, "**** int4 size defined wrong in etsheadr.h! Aborting\n");
    exit(1);
  }
if (strlen(datafile->filename) == 0)  /* no predefined name */
  {
    if (datafile->this_machine == IEEE_MACHINE)
      printf("This is an IEEE format machine, i.e. RISC, 680x0, etc.\n");
    else
      printf("This is a non-IEEE format machine such as DEC or 80x86\n");
  }

header_char = (char *) &slbd_hdr;
sprintf(inp_frmt, "%%%dc", 20);  /* should be "%20c" */
/*************************** open file *************/
do
  {
    if (strlen(datafile->filename) == 0)  /* no predefined name */
      {
	printf("enter file name: ");
	scanf("%[^\n]", datafile->filename);
	getchar();
	printf("data file: %s\n\n", datafile->filename);
      }
    datafile->dat_file = fopen(datafile->filename, "r");
    if (datafile->dat_file == NULL)
      {
	fprintf(stderr, "Smooth move, wing nut! Now you've really goobered it up!\n");
	fprintf(stderr, "Unable to open file %s, try again\n",
		datafile->filename);
	datafile->filename[0] = '\0';  /* reset for loop back & try again */
/*	fprintf(stderr, "You leave me no choice but to BARF OUT. Bye bye\n\n");
	exit(1);*/
      }
  }
while (datafile->dat_file == NULL);
fscanf(datafile->dat_file, inp_frmt, header_char);
datafile->current_frame = 0;

/************ determine byte ordering *****************/
if (datafile->this_machine == NON_IEEE_MACHINE)
  datafile->data_order = PROPER_ORDER;
else
  datafile->data_order = REVERSE_ORDER;

/************* swap bytes if necessary ********/
if (datafile->data_order == REVERSE_ORDER)
   /* swap bytes if IEEE machine reading non-IEEE data or vice-versa */
  {
    printf("swapping header bytes\n");
    swab(&header_char[0], tbuf, 20);  /* 5 32 bit words */
    memcpy(&header_char[0], tbuf, 20);
  }

/************** move stuff to ets header *******************/
datafile->file_header.major_version = 0;
datafile->file_header.minor_version = 0;
datafile->file_header.bits_per_pixel = 16;
datafile->file_header.chipnum = 0;
datafile->file_header.col_start = 0;
datafile->file_header.row_start = 0;

nframes = slbd_hdr.end_frame - slbd_hdr.start_frame + 1;
datafile->file_header.ext_hdr_len = slbd_hdr.frm_len;
datafile->file_header.col_length = slbd_hdr.nrows;
datafile->file_header.row_length = slbd_hdr.ncols;
datafile->file_header.frames_requested = nframes;
datafile->file_header.frames_good = nframes;
datafile->file_header.big_little = PROPER_ORDER;
datafile->file_header.frame_type = STANDARD_2BYTE_INT;
strcpy(datafile->file_header.description, "slbd data.");
strcpy(datafile->file_header.proc_hist, "");
strcpy(datafile->file_header.operator, "");
strcpy(datafile->file_header.start_time, "");
strcpy(datafile->file_header.end_time, "");

/*printf("start frame = %d; end frame = %d\n",
       slbd_hdr.start_frame, slbd_hdr.end_frame);*/

/******** position to start of first frame ************/
fseek(datafile->dat_file, datafile->file_header.ext_hdr_len*2, 0);
}

/******************* read a frame of data ********************/
#ifdef NON_ANSI
int2 * read_slbd_frame(datafile, frame_num)
     ETS_STRUCT
       *datafile;
     int
       frame_num; /* if 0, then next frame, else get given frame # */
#else
int2 * read_slbd_frame
  (
   ETS_STRUCT * datafile,
   int frame_num   /* if 0, then next frame, else get given frame_num */
  )
#endif
{
    int2
      ncols,
      nrows,
      nframes,
      *pix_array;
    int
      i,
      j,
      k,
      maxbytes;
    char
      *tbuf1,
      tbuf2[20],
      inp_frmt[10],
      hdr_frmt[10],
      header_rec[HEADER_SIZE],
      *input_data;
    struct
      {
	int2 fnum;
	int2 fill1;
	int2 fill2;
	int2 fill3;
      } slbd_anot;

ncols = datafile->file_header.row_length;  /* Note that # pix/row = # cols */
nrows = datafile->file_header.col_length;
nframes = datafile->file_header.frames_good;
maxbytes = sizeof(int2) * ncols;

input_data = (char *) calloc(maxbytes, sizeof(char));
if (input_data == NULL)
  {
    fprintf(stderr, "calloc failure for input_data\n");
    exit(1);
  }
sprintf(inp_frmt, "%%%dc", maxbytes); /* should be, e.g. "%840c" */
pix_array = (int2 *) calloc(nrows*ncols, sizeof(int2));
if (pix_array == NULL)
  {
   fprintf(stderr, "Failure to malloc pix_array in readframe in filefunc.c\n");
   fprintf(stderr, "aborting ...\n");
   exit(1);
  }

/******************* read in next frame *********************/
if (frame_num == 0)
  {
  if (datafile->current_frame >= nframes)
    {
      return(NULL);
    }
  else
    {
      ++datafile->current_frame;
      for (i=0; i<nrows; ++i)
	{
	  fscanf(datafile->dat_file, inp_frmt, input_data);  /* read a line */
	  if (datafile->data_order == REVERSE_ORDER)
	    swab(input_data, (pix_array + i*ncols), maxbytes); /* swap bytes */
	  else
	    memcpy((pix_array + i*ncols), input_data, maxbytes);/*no swapping*/
	} /* end i-loop for nrows */
    } /* end, still in good frame region */
  } /* (end if frame_num == 0) */
else /* move to  frame_num */
  {
    fseek(datafile->dat_file,
	  (frame_num) * datafile->file_header.ext_hdr_len * 2, 0);
    datafile->current_frame = frame_num;  /* base 1 */
    /**** okay, now read in the frame ****/
    for (i=0; i<nrows; ++i)
      {
	  fscanf(datafile->dat_file, inp_frmt, input_data);  /* read a line */
	  if (datafile->data_order == REVERSE_ORDER)
	    swab(input_data, (pix_array + i*ncols), maxbytes); /* swap bytes */
	  else
	    memcpy((pix_array + i*ncols), input_data, maxbytes);/*no swapping*/
      } /* end frame */
  } /* end single frame processing */

/********** get last few bytes **********/
tbuf1 = (char *) &slbd_anot;
fscanf(datafile->dat_file, "%8c", tbuf1);
if (datafile->data_order == REVERSE_ORDER)
  {
    swab(&tbuf1[0], tbuf2, 8);  /* 4 2 bit words */
    memcpy(&tbuf1[0], tbuf2, 8);
  }
datafile->file_header.chipnum = slbd_anot.fnum;

/*printf("fnum=%d; fill1=%d; fill2=%d; fill3=%d\n", slbd_anot.fnum,
       slbd_anot.fill1, slbd_anot.fill2, slbd_anot.fill3);*/

free(input_data);
return(pix_array);
}

/****************** close file *********************/
#ifdef NON_ANSI
void slbdclose(dat_file)
     FILE
       *dat_file;
#else
void slbdclose
  (
   FILE * dat_file
  )
#endif
{
fclose(dat_file);
}
