/**********************************************************************
*
*                                  xgm.c
*
*  Copyright (c) 1992, John N. Sanders, Massachusetts Institute of
*      Technology, Lincoln Laboratory; 244 Wood Street, Lexington,
*      MA 02173.
*  All Rights Reserved.
*  Version 2.0   4/93
*
*  Shell Program for XgMovie
*
***********************************************************************
*/

#include "stdio.h"
#include "string.h"
#include "math.h"

#include "X11/Intrinsic.h"
#include "X11/Shell.h"
#include "Xm/Xm.h"

#include "etsheadr.h"
#include "XgMovie.h"

int
  image_format;

/*************************************************************/
main(argc, argv)
     int
       argc;
     char
       **argv;
{
  ETS_STRUCT
    datafile;
  XgMovie_data
    *mv_data;
  int
    i,
    all_frames, /* flag set by command line to indicate display all frames */
    proc_frames,
    start,
    stop,
    nrows,
    ncols,
    nframes,
    npixels;
  int2
    *temp,
    *pix_array;
  char
    buf[160];
  Display
    *display;
  XtAppContext
    theAppContext;

/***************** initial setup *************/
image_format = 0;
datafile.filename[0] = '\0';  /* make sure we don't open garbage */
all_frames = 0;
for (i=1; i<argc; ++i) /* find file name if given */
  {
    if (!strncmp(argv[i], "-h", 2))  /** display basic usage message **/
      {
	XgM_Command_Help();
	exit(0);
      }
    else if (!strncmp(argv[i], "-s", 2))  /** select SLBD format data **/
      {
	image_format = 1;
	printf("Selected SLBD format data\n");
      }
    else if (!strncmp(argv[i], "-a", 2))  /** automatically use all frames **/
      {
	all_frames = 1;
      }
    else if (!strncmp(argv[i], "-", 1)) /*if this is - option, skip next arg */
      ++i;
    else /** assume this is file name **/
      {
	strcpy(datafile.filename, argv[i]);
      }
  }
if (image_format == 1)
  slbdopen(&datafile);
else
  fileopen(&datafile);
/*showhead(&datafile);*/
printf("Data Description: %s\n", datafile.file_header.description);
printf("Processing History: %s\n", datafile.file_header.proc_hist);
nrows = datafile.file_header.col_length;
ncols = datafile.file_header.row_length;
nframes = datafile.file_header.frames_good;
npixels = nrows * ncols;

printf("xgm: There are %d frames of %d x %d (width x height) data.\n",
       nframes, ncols, nrows);
if (all_frames == 1)
  {
    printf("Using all frames\n");
    start = 1;
    stop = nframes;
  }
else  /** ask for start, stop frames **/
  {
    do
      {
	printf("At what frame do you wish to start (base 1)? ");
	scanf("%d", &start);
	getchar();
      }
    while ((start < 1) || (start > nframes));
    do
      {
	printf("At what frame (inclusive) do you wish to stop? ");
	scanf("%d", &stop);
	getchar();
      }
    while ((stop < start) || (stop > nframes));
  }

proc_frames = stop - start + 1;
pix_array = (int2 *) calloc(npixels*proc_frames, sizeof(int2));
if (pix_array == NULL)
  {
    fprintf(stderr, "Failure to calloc pix_array in xgm.c. Aborting\n");
    exit(1);
  }

/****************** read in frames ****************/
/*** do first frame ***/
printf("***** reading frame %d\r", start);
fflush(stdout);
if (image_format == 1)
  temp = read_slbd_frame(&datafile, start);  /* sets frame pointer */
else
  temp = readframe(&datafile, start);  /* sets frame pointer */
memcpy(pix_array, temp, sizeof(int2)*npixels);
free(temp);
/*** do rest of frames ***/
for (i=start; i<stop; ++i)
  {
    printf("***** reading frame %d\r", i+1);
    fflush(stdout);
    if (image_format == 1)
      temp = read_slbd_frame(&datafile, 0);
    else
      temp = readframe(&datafile, 0);
    memcpy(pix_array + (i-start+1)*npixels, temp, sizeof(int2)*npixels);
    free(temp);
  }
printf("\n");
fileclose(datafile.dat_file);  /** finished with incoming data **/

/************ connect to X-Windows & start movie ***************/
theAppContext = NULL;
display = NULL;

XtToolkitInitialize();
theAppContext = XtCreateApplicationContext();
if (theAppContext == NULL)
  {
    fprintf(stderr, "Unable to create application context. Aborting\n");
    exit(1);
  }
display = XtOpenDisplay(theAppContext, NULL, argv[0], "XgMovie", NULL, 0,
			(Cardinal *) &argc, argv);
if (display == NULL)
  {
    fprintf(stderr, "Unable to open requested display. Aborting\n");
    exit(1);
  }

mv_data = XgM_Initialize_Movie();
mv_data->nrows = nrows;
mv_data->ncols = ncols;
mv_data->nframes = proc_frames;
mv_data->xoff = 0;
mv_data->yoff = 0;
mv_data->pix_array = pix_array;
strcpy(mv_data->filename, datafile.filename);
mv_data->theAppContext = theAppContext;
mv_data->display = display;

XgM_Main_Entry(mv_data);
XtAppMainLoop(theAppContext);
}

/*********************************************/
XgM_Command_Help()
{
fprintf(stderr, "xgm command line help:\n");
fprintf(stderr, " usage: xgm [options]\n");
fprintf(stderr, "      where options may be one or more of the following:\n");
fprintf(stderr, "   any of the standard Xt command line options\n");
fprintf(stderr, "   [(an image file name)] indicating the image file to display\n");
fprintf(stderr, "     The image file format is described in etsheadr.h\n");
fprintf(stderr, "   [-s] read SLBD format image data instead of ETS\n");
fprintf(stderr, "   [-a] to automatically use all frames of data\n");
fprintf(stderr, "   [-h] this message\n");
fprintf(stderr, "  This program requires an 8-bit pseudocolor (adjustable)\n");
fprintf(stderr, "    visual to run. You should also have enough memory to\n");
fprintf(stderr, "    hold the entire movie data set in memory.\n");
}
